from insights.parsers.ls import (
    LSla, LSlaFiltered, LSlan, LSlanFiltered,
    LSlanL, LSlanR, LSlanRL, LSlaRZ, LSlaZ, LSldH, LSldZ
)
from insights.tests import context_wrap

LS_LA = """
ls: cannot access '_non_existing_': No such file or directory
/boot:
total 187380
dr-xr-xr-x.  3 root root     4096 Mar  4 16:19 .
dr-xr-xr-x. 19 root root     4096 Jul 14 09:10 ..
-rw-r--r--.  1 root root   123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64
drwxr-xr-x.  6 root root      111 Jun 15 09:51 grub2

/boot/grub2:
total 36
drwxr-xr-x. 6 root root  104 Mar  4 16:16 .
dr-xr-xr-x. 3 root root 4096 Mar  4 16:19 ..
lrwxrwxrwx. 1 root root   11 Aug  4  2014 menu.lst -> ./grub.cfg
-rw-r--r--. 1 root root   64 Sep 18  2015 device.map
-rw-r--r--. 1 root root 7040 Mar 29 13:30 grub.cfg
"""

LS_LAN = """
/boot:
total 187380
dr-xr-xr-x.  3 0 0     4096 Mar  4 16:19 .
dr-xr-xr-x. 19 0 0     4096 Jul 14 09:10 ..
-rw-r--r--.  1 0 0   123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64
drwxr-xr-x.  6 0 0      111 Jun 15 09:51 grub2

/boot/grub2:
total 36
drwxr-xr-x. 6 0 0  104 Mar  4 16:16 .
dr-xr-xr-x. 3 0 0 4096 Mar  4 16:19 ..
lrwxrwxrwx. 1 0 0   11 Aug  4  2014 menu.lst -> ./grub.cfg
-rw-r--r--. 1 0 0   64 Sep 18  2015 device.map
-rw-r--r--. 1 0 0 7040 Mar 29 13:30 grub.cfg
"""


LS_LANL = """
ls: cannot access '_non_existing_': No such file or directory
/boot:
total 187380
dr-xr-xr-x.  3 0 0     4096 Mar  4 16:19 .
dr-xr-xr-x. 19 0 0     4096 Jul 14 09:10 ..
-rw-r--r--.  1 0 0   123891 Aug 25  2015 config-3.10.0-229.14.1.el7.x86_64
drwxr-xr-x.  6 0 0      111 Jun 15 09:51 grub2

/boot/grub2:
total 32
drwx------. 4 0 0   83 Jun 27 11:01 .
dr-xr-xr-x. 5 0 0 4096 Jun  5 09:14 ..
-rw-r--r--. 1 0 0   11 Aug  4  2014 menu.lst
-rw-r--r--. 1 0 0   64 Jan 17 16:10 device.map
-rw-------. 1 0 0 6529 Jan 17 16:10 grub.cfg
"""

LS_LANR = """
ls: cannot access '_non_existing_': No such file or directory
/dev:
total 0
drwxr-xr-x. 21 0  0     3100 Jun 27 10:58 .
dr-xr-xr-x. 17 0  0      224 Apr 17 16:45 ..
crw-r--r--.  1 0  0  10, 235 Jun 27 10:58 autofs
drwxr-xr-x.  2 0  0      160 Jun 27 10:58 block
lrwxrwxrwx.  1 0  0        3 Jun 27 10:58 cdrom -> sr0

/dev/vfio:
total 0
drwxr-xr-x.  2 0 0      60 Jun 27 10:58 .
drwxr-xr-x. 21 0 0    3100 Jun 27 10:58 ..
crw-------.  1 0 0 10, 196 Jun 27 10:58 vfio

/dev/virtio-ports:
total 0
drwxr-xr-x.  2 0 0   80 Jun 27 10:58 .
drwxr-xr-x. 21 0 0 3100 Jun 27 10:58 ..
lrwxrwxrwx.  1 0 0   11 Jun 27 10:58 com.redhat.spice.0 -> ../vport2p2
lrwxrwxrwx.  1 0 0   11 Jun 27 10:58 org.qemu.guest_agent.0 -> ../vport2p1
"""

LS_LANRL = """
/dev:
total 0
drwxr-xr-x. 21    0  0            3100 Jun 27 10:58 .
dr-xr-xr-x. 17    0  0             224 Apr 17 16:45 ..
crw-r--r--.  1    0  0         10, 235 Jun 27 10:58 autofs
drwxr-xr-x.  2    0  0             160 Jun 27 10:58 block
brw-rw----.  1    0 11         11,   0 Jun 27 10:58 cdrom

/dev/vfio:
total 0
drwxr-xr-x.  2 0 0      60 Jun 27 10:58 .
drwxr-xr-x. 21 0 0    3100 Jun 27 10:58 ..
crw-------.  1 0 0 10, 196 Jun 27 10:58 vfio

/dev/virtio-ports:
total 0
drwxr-xr-x.  2 0 0     80 Jun 27 10:58 .
drwxr-xr-x. 21 0 0   3100 Jun 27 10:58 ..
crw-------.  1 0 0 243, 2 Jun 27 10:58 com.redhat.spice.0
crw-------.  1 0 0 243, 1 Jun 27 10:58 org.qemu.guest_agent.0
"""

LS_LANRZ = """
/dev:
total 0
drwxr-xr-x. 21 0  0 system_u:object_r:device_t:s0                  3100 Jun 27 10:58 .
dr-xr-xr-x. 17 0  0 system_u:object_r:root_t:s0                     224 Apr 17 16:45 ..
crw-r--r--.  1 0  0 system_u:object_r:autofs_device_t:s0            235 Jun 27 10:58 autofs
drwxr-xr-x.  2 0  0 system_u:object_r:device_t:s0                   160 Jun 27 10:58 block
lrwxrwxrwx.  1 0  0 system_u:object_r:device_t:s0                     3 Jun 27 10:58 cdrom -> sr0

/dev/vfio:
total 0
drwxr-xr-x.  2 0 0 system_u:object_r:device_t:s0           60 Jun 27 10:58 .
drwxr-xr-x. 21 0 0 system_u:object_r:device_t:s0         3100 Jun 27 10:58 ..
crw-------.  1 0 0 system_u:object_r:vfio_device_t:s0     196 Jun 27 10:58 vfio

/dev/virtio-ports:
total 0
drwxr-xr-x.  2 0 0 system_u:object_r:device_t:s0   80 Jun 27 10:58 .
drwxr-xr-x. 21 0 0 system_u:object_r:device_t:s0 3100 Jun 27 10:58 ..
lrwxrwxrwx.  1 0 0 system_u:object_r:device_t:s0   11 Jun 27 10:58 com.redhat.spice.0 -> ../vport2p2
lrwxrwxrwx.  1 0 0 system_u:object_r:device_t:s0   11 Jun 27 10:58 org.qemu.guest_agent.0 -> ../vport2p1
"""

LS_LARZ_CONTENT1 = """
/var/log:
total 13500
drwxr-xr-x. 16 root   root   system_u:object_r:var_log_t:s0                    4096 Oct 15 00:00 .
drwxr-xr-x. 20 root   root   system_u:object_r:var_t:s0                        4096 May 27  2022 ..
drwxr-xr-x.  2 root   root   system_u:object_r:var_log_t:s0                    4096 May 27  2022 anaconda
drwx------.  2 root   root   system_u:object_r:auditd_log_t:s0                   99 Oct 16 04:58 audit
-rw-------.  1 root   root   system_u:object_r:plymouthd_var_log_t:s0             0 May 11 00:00 boot.log
-rw-r--r--.  1 root   root   unconfined_u:object_r:rpm_log_t:s0                5220 Sep 23 23:44 hawkey.log-20230924
-rw-r--r--.  1 root   root   unconfined_u:object_r:rpm_log_t:s0                5160 Sep 30 21:40 hawkey.log-20231001
-rw-r--r--.  1 root   root   unconfined_u:object_r:rpm_log_t:s0                5340 Oct  7 23:50 hawkey.log-20231008
-rw-r--r--.  1 root   root   unconfined_u:object_r:rpm_log_t:s0                5640 Oct 14 23:26 hawkey.log-20231015
drwx------.  2 root   root   system_u:object_r:insights_client_var_log_t:s0     120 Oct 17 00:17 insights-client
-rw-r--r--.  1 root   root   system_u:object_r:var_log_t:s0                    8062 May 10 09:55 kdump.log -> false-link.log

/var/log/anaconda:
total 5228
drwxr-xr-x.  2 root root system_u:object_r:var_log_t:s0    4096 May 27  2022 .
drwxr-xr-x. 16 root root system_u:object_r:var_log_t:s0    4096 Oct 15 00:00 ..
-rw-------.  1 root root system_u:object_r:var_log_t:s0   46971 May 27  2022 anaconda.log
-rw-------.  1 root root system_u:object_r:var_log_t:s0    3641 May 27  2022 dbus.log
-rw-------.  1 root root system_u:object_r:var_log_t:s0    1799 May 27  2022 dnf.librepo.log
-rw-------.  1 root root system_u:object_r:var_log_t:s0     120 May 27  2022 hawkey.log
-rw-------.  1 root root system_u:object_r:var_log_t:s0 3519082 May 27  2022 journal.log
-rw-------.  1 root root system_u:object_r:var_log_t:s0      40 May 27  2022 ks-script-d2lfa1o_.log
-rw-------.  1 root root system_u:object_r:var_log_t:s0      63 May 27  2022 ks-script-r_i_73z0.log
-rw-------.  1 root root system_u:object_r:var_log_t:s0       0 May 27  2022 ks-script-w3x_f4hl.log
"""

LS_LARZ_CONTENT2 = """
/var/log/rhsm:
drwxr-xr-x. root root system_u:object_r:rhsmcertd_log_t:s0 .
drwxr-xr-x. root root system_u:object_r:var_log_t:s0   ..
-rw-r--r--. root root unconfined_u:object_r:rhsmcertd_log_t:s0 rhsm.log
-rw-r--r--. root root unconfined_u:object_r:rhsmcertd_log_t:s0 rhsm.log-20231015
-rw-r--r--. root root system_u:object_r:rhsmcertd_log_t:s0 rhsmcertd.log
-rw-r--r--. root root system_u:object_r:rhsmcertd_log_t:s0 rhsmcertd.log-20231015

/var/log/sa:
drwxr-xr-x. root root system_u:object_r:sysstat_log_t:s0 .
drwxr-xr-x. root root system_u:object_r:var_log_t:s0   ..
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa12
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa13
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa14
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa15
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa16
-rw-r--r--  root root ?                                sa17
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sar12
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sar13
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sar14
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sar15
-rw-r--r--  root root ?                                sar16
"""

LS_LANZ = """
/dev:
total 0
drwxr-xr-x. 21 0  0 system_u:object_r:device_t:s0          3100 Jun 27 10:58 .
dr-xr-xr-x. 17 0  0 system_u:object_r:root_t:s0             224 Apr 17 16:45 ..
crw-r--r--.  1 0  0 system_u:object_r:autofs_device_t:s0    235 Jun 27 10:58 autofs
drwxr-xr-x.  2 0  0 system_u:object_r:device_t:s0           160 Jun 27 10:58 block
lrwxrwxrwx.  1 0  0 system_u:object_r:device_t:s0             3 Jun 27 10:58 cdrom -> sr0
"""

LS_LAZ_CONTENT1 = """
/var/log:
total 13500
drwxr-xr-x. 16 root   root   system_u:object_r:var_log_t:s0                    4096 Oct 15 00:00 .
drwxr-xr-x. 20 root   root   system_u:object_r:var_t:s0                        4096 May 27  2022 ..
drwxr-xr-x.  2 root   root   system_u:object_r:var_log_t:s0                    4096 May 27  2022 anaconda
drwx------.  2 root   root   system_u:object_r:auditd_log_t:s0                   99 Oct 16 04:58 audit
-rw-------.  1 root   root   system_u:object_r:plymouthd_var_log_t:s0             0 May 11 00:00 boot.log
-rw-r--r--.  1 root   root   unconfined_u:object_r:rpm_log_t:s0                5220 Sep 23 23:44 hawkey.log-20230924
-rw-r--r--.  1 root   root   unconfined_u:object_r:rpm_log_t:s0                5160 Sep 30 21:40 hawkey.log-20231001
-rw-r--r--.  1 root   root   unconfined_u:object_r:rpm_log_t:s0                5340 Oct  7 23:50 hawkey.log-20231008
-rw-r--r--.  1 root   root   unconfined_u:object_r:rpm_log_t:s0                5640 Oct 14 23:26 hawkey.log-20231015
drwx------.  2 root   root   system_u:object_r:insights_client_var_log_t:s0     120 Oct 17 00:17 insights-client
-rw-r--r--.  1 root   root   system_u:object_r:var_log_t:s0                    8062 May 10 09:55 kdump.log
"""

LS_LAZ_CONTENT2 = """
/var/log/rhsm:
drwxr-xr-x. root root system_u:object_r:rhsmcertd_log_t:s0 .
drwxr-xr-x. root root system_u:object_r:var_log_t:s0   ..
-rw-r--r--. root root unconfined_u:object_r:rhsmcertd_log_t:s0 rhsm.log
-rw-r--r--. root root unconfined_u:object_r:rhsmcertd_log_t:s0 rhsm.log-20231015
-rw-r--r--. root root system_u:object_r:rhsmcertd_log_t:s0 rhsmcertd.log
-rw-r--r--. root root system_u:object_r:rhsmcertd_log_t:s0 rhsmcertd.log-20231015

/var/log/sa:
drwxr-xr-x. root root system_u:object_r:sysstat_log_t:s0 .
drwxr-xr-x. root root system_u:object_r:var_log_t:s0   ..
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa12
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa13
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa14
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa15
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sa16
-rw-r--r--  root root ?                                sa17
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sar12
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sar13
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sar14
-rw-r--r--. root root system_u:object_r:sysstat_log_t:s0 sar15
-rw-r--r--  root root ?                                sar16

/dev:
crw-rw----. root root    system_u:object_r:clock_device_t:s0 rtc0
drwxrwxrwt. root root    system_u:object_r:tmpfs_t:s0     shm
crw-rw----. root root    system_u:object_r:apm_bios_t:s0  snapshot
lrwxrwxrwx. root root    system_u:object_r:device_t:s0    stderr -> /proc/self/fd/2
lrwxrwxrwx. root root    system_u:object_r:device_t:s0    stdin -> /proc/self/fd/0
lrwxrwxrwx. root root    system_u:object_r:device_t:s0    stdout -> /proc/self/fd/1
lrwxrwxrwx. root root    system_u:object_r:device_t:s0    systty -> tty0
"""

LS_LAZ_CONTENT3_RHEL8_SELINUX = """
/etc:
total 1352
drwxr-xr-x. 133 root root   system_u:object_r:etc_t:s0                      12288 Nov 12 23:55 .
dr-xr-xr-x.  21 root root   system_u:object_r:root_t:s0                      4096 Jul  5  2022 ..
-rw-r--r--    1 root root   ?                                                1529 Apr 15  2020 aliases
-rw-r--r--    1 root root   ?                                               12288 Jul  5  2022 aliases.db
-rw-r--r--.   1 root root   system_u:object_r:etc_t:s0                          1 Aug 12  2018 at.deny
drwxr-x---.   4 root root   system_u:object_r:auditd_etc_t:s0                 150 Nov 12 23:55 audit
drwxr-xr-x.   2 root root   system_u:object_r:etc_t:s0                        160 Sep 15 12:32 bash_completion.d
-rw-r--r--    1 root root   ?                                                3019 Apr 15  2020 bashrc
-rw-r--r--    1 root root   ?                                                 881 Jul  5  2022 chrony.conf
-rw-r-----.   1 root chrony system_u:object_r:chronyd_keys_t:s0               540 May 10  2019 chrony.keys
drwxr-xr-x.   2 root root   system_u:object_r:etc_t:s0                         26 Jul 28  2021 cifs-utils

/var/log:
total 114496
drwxr-xr-x. 17 root   root   system_u:object_r:var_log_t:s0               8192 Nov 12 03:47 .
drwxr-xr-x. 23 root   root   system_u:object_r:var_t:s0                   4096 Jul  5  2022 ..
drwxr-xr-x   3 root   root   ?                                             122 Jul  5  2022 Automation
drwxr-xr-x.  2 root   root   system_u:object_r:var_log_t:s0                280 Mar 11  2020 anaconda
drwx------.  2 root   root   system_u:object_r:auditd_log_t:s0              99 Nov 12 19:00 audit
-rw-------.  1 root   root   system_u:object_r:plymouthd_var_log_t:s0        0 Jul  5  2022 boot.log
-rw-------   1 root   root   ?                                           79952 Sep 30  2020 boot.log-20200930
-rw-------   1 root   root   ?                                           22027 Jul  5  2022 boot.log-20220705
-rw-------   1 root   utmp   ?                                               0 Nov  1 03:15 btmp
drwxr-xr-x.  2 chrony chrony system_u:object_r:chronyd_var_log_t:s0          6 Mar  1  2021 chrony
""".strip()


LS_LARZ_DEV_CONTENT1 = """
/dev:
total 0
drwxr-xr-x. 19 root root    system_u:object_r:device_t:s0                  3180 May 10 09:54 .
dr-xr-xr-x. 18 root root    system_u:object_r:root_t:s0                     235 May 27  2022 ..
crw-r--r--.  1 root root    system_u:object_r:autofs_device_t:s0        10, 235 May 10 09:54 autofs
drwxr-xr-x.  2 root root    system_u:object_r:device_t:s0                   140 Jun 15 16:11 block
drwxr-xr-x.  3 root root    system_u:object_r:device_t:s0                    60 May 10 09:54 bus
drwxr-xr-x.  2 root root    system_u:object_r:device_t:s0                  2680 May 10 09:55 char
crw--w----.  1 root tty     system_u:object_r:console_device_t:s0        5,   1 May 10 09:54 console

/dev/vfio:
total 0
drwxr-xr-x.  2 root root system_u:object_r:device_t:s0           60 May 10 09:54 .
drwxr-xr-x. 19 root root system_u:object_r:device_t:s0         3180 May 10 09:54 ..
crw-rw-rw-.  1 root root system_u:object_r:vfio_device_t:s0 10, 196 May 10 09:54 vfio -> false_link_2
"""

LS_LDH = """
ls: cannot access '/dev/socketfile': No such file or directory
dr-xr-xr-x. 5 root root     4096 May 30 06:57 /boot
drwx------. 4 root root       83 Nov  6  2024 /boot/grub2
-rw-------. 1 root root     6658 Dec 20  2023 /boot/grub2/grub.cfg
crw--w----. 1 root tty  136,   0 Nov 17 01:28 /dev/stderr
brw-rw----. 1 root disk 252,   0 Nov 16 03:19 /dev/vda
crw-------. 1 root root  10, 137 Nov  6  2024 /dev/vhci
"""

LS_LDZ = """
dr-xr-xr-x. 5 root root system_u:object_r:boot_t:s0                  4096 May 30 06:57 /boot
-rw-------. 1 root root system_u:object_r:boot_t:s0                  6658 Dec 20  2023 /boot/grub2/grub.cfg
lrwxrwxrwx. 1 root root system_u:object_r:device_t:s0                  15 Nov  6  2024 /dev/stderr -> /proc/self/fd/2
brw-rw----. 1 root disk system_u:object_r:fixed_disk_device_t:s0 252,   0 Nov  7 09:51 /dev/vda
crw-------. 1 root root system_u:object_r:vhost_device_t:s0       10, 137 Nov  6  2024 /dev/vhci
-rw-------. 1 root root system_u:object_r:var_log_t:s0             275652 Nov  7 10:20 /var/log/messages
"""


def test_ls_la():
    ls = LSla(context_wrap(LS_LA))
    assert '/boot' in ls
    assert '/boot/grub2' in ls
    assert ls.dirs_of('/boot') == ['.', '..', 'grub2']

    grub2_files = ls.files_of('/boot/grub2')
    assert 'menu.lst' in grub2_files
    assert 'device.map' in grub2_files
    assert 'grub.cfg' in grub2_files
    assert len(grub2_files) == 3

    boot_files = ls.files_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_files
    assert len(boot_files) == 1

    boot_listings = ls.listing_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_listings
    assert boot_listings["config-3.10.0-229.14.1.el7.x86_64"]['owner'] == 'root'

    grub_listings = ls.listing_of('/boot/grub2')
    assert "menu.lst" in grub_listings
    assert grub_listings["menu.lst"]['group'] == 'root'
    assert 'link' in grub_listings["menu.lst"]
    assert grub_listings["menu.lst"]['link'] == './grub.cfg'


def test_ls_la_filtered():
    ls = LSlaFiltered(context_wrap(LS_LA))
    assert '/boot' in ls
    assert '/boot/grub2' in ls
    assert ls.dirs_of('/boot') == ['.', '..', 'grub2']

    grub2_files = ls.files_of('/boot/grub2')
    assert 'menu.lst' in grub2_files
    assert 'device.map' in grub2_files
    assert 'grub.cfg' in grub2_files
    assert len(grub2_files) == 3

    boot_files = ls.files_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_files
    assert len(boot_files) == 1

    boot_listings = ls.listing_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_listings
    assert boot_listings["config-3.10.0-229.14.1.el7.x86_64"]['owner'] == 'root'

    grub_listings = ls.listing_of('/boot/grub2')
    assert "menu.lst" in grub_listings
    assert grub_listings["menu.lst"]['group'] == 'root'
    assert 'link' in grub_listings["menu.lst"]
    assert grub_listings["menu.lst"]['link'] == './grub.cfg'


def test_ls_lan():
    ls = LSlan(context_wrap(LS_LAN))
    assert '/boot' in ls
    assert '/boot/grub2' in ls
    assert ls.dirs_of('/boot') == ['.', '..', 'grub2']

    grub2_files = ls.files_of('/boot/grub2')
    assert 'menu.lst' in grub2_files
    assert 'device.map' in grub2_files
    assert 'grub.cfg' in grub2_files
    assert len(grub2_files) == 3

    boot_files = ls.files_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_files
    assert len(boot_files) == 1

    boot_listings = ls.listing_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_listings
    assert boot_listings["config-3.10.0-229.14.1.el7.x86_64"]['owner'] == '0'

    grub_listings = ls.listing_of('/boot/grub2')
    assert "menu.lst" in grub_listings
    assert grub_listings["menu.lst"]['group'] == '0'
    assert 'link' in grub_listings["menu.lst"]
    assert grub_listings["menu.lst"]['link'] == './grub.cfg'


def test_ls_lan_filtered():
    ls = LSlanFiltered(context_wrap(LS_LAN))
    assert '/boot' in ls
    assert '/boot/grub2' in ls
    assert ls.dirs_of('/boot') == ['.', '..', 'grub2']

    grub2_files = ls.files_of('/boot/grub2')
    assert 'menu.lst' in grub2_files
    assert 'device.map' in grub2_files
    assert 'grub.cfg' in grub2_files
    assert len(grub2_files) == 3

    boot_files = ls.files_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_files
    assert len(boot_files) == 1

    boot_listings = ls.listing_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_listings
    assert boot_listings["config-3.10.0-229.14.1.el7.x86_64"]['owner'] == '0'

    grub_listings = ls.listing_of('/boot/grub2')
    assert "menu.lst" in grub_listings
    assert grub_listings["menu.lst"]['group'] == '0'
    assert 'link' in grub_listings["menu.lst"]
    assert grub_listings["menu.lst"]['link'] == './grub.cfg'


def test_ls_lanL():
    ls = LSlanL(context_wrap(LS_LANL))
    assert '/boot' in ls
    assert '/boot/grub2' in ls
    assert ls.dirs_of('/boot') == ['.', '..', 'grub2']

    grub2_files = ls.files_of('/boot/grub2')
    assert 'menu.lst' in grub2_files
    assert 'device.map' in grub2_files
    assert 'grub.cfg' in grub2_files
    assert len(grub2_files) == 3

    boot_files = ls.files_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_files
    assert len(boot_files) == 1

    boot_listings = ls.listing_of('/boot')
    assert "config-3.10.0-229.14.1.el7.x86_64" in boot_listings
    assert boot_listings["config-3.10.0-229.14.1.el7.x86_64"]['owner'] == '0'

    grub_listings = ls.listing_of('/boot/grub2')
    assert "menu.lst" in grub_listings
    assert grub_listings["menu.lst"]['group'] == '0'
    assert 'link' not in grub_listings["menu.lst"]


def test_ls_lanR():
    ls = LSlanR(context_wrap(LS_LANR))
    assert '/dev' in ls
    assert '/dev/vfio' in ls
    assert '/dev/virtio-ports' in ls
    assert ls.dirs_of('/dev') == ['.', '..', 'block']

    dev_listings = ls.listing_of('/dev')
    assert "cdrom" in dev_listings
    assert 'link' in dev_listings["cdrom"]
    assert dev_listings["cdrom"]['link'] == 'sr0'

    assert 'link' in ls.listing_of('/dev/virtio-ports')['com.redhat.spice.0']

    assert ls.files_of('non-exist') == []
    assert ls.dirs_of('non-exist') == []
    assert ls.specials_of('non-exist') == []
    assert ls.listing_of('non-exist') == {}
    assert ls.total_of('non-exist') == 0
    assert ls.dir_contains('non-exist', 'test') is False
    assert ls.dir_entry('non-exist', 'test') == {}


def test_ls_lanRL():
    ls = LSlanRL(context_wrap(LS_LANRL))
    assert '/dev' in ls
    assert '/dev/vfio' in ls
    assert '/dev/virtio-ports' in ls
    assert ls.dirs_of('/dev') == ['.', '..', 'block']

    dev_listings = ls.listing_of('/dev')
    assert "cdrom" in dev_listings
    assert 'link' not in dev_listings["cdrom"]

    assert 'link' not in ls.listing_of('/dev/virtio-ports')['com.redhat.spice.0']


def test_ls_laRZ():
    ls = LSlaRZ(context_wrap(LS_LARZ_CONTENT1))
    assert '/var/log' in ls
    assert '/var/log/anaconda' in ls
    assert '/var/log/atest' not in ls
    assert ls.dirs_of('/var/log') == ['.', '..', 'anaconda', 'audit', 'insights-client']

    dev_listings = ls.listing_of('/var/log')
    assert 'boot.log' in dev_listings
    assert dev_listings["boot.log"]['se_type'] == 'plymouthd_var_log_t'
    assert 'kdump.log' in dev_listings
    assert dev_listings["kdump.log"]['link'] == 'false-link.log'

    ls = LSlaRZ(context_wrap(LS_LARZ_CONTENT2))
    assert '/var/log/rhsm' in ls

    dev_listings = ls.listing_of('/var/log/rhsm')
    assert "rhsm.log" in dev_listings
    assert 'se_type' in dev_listings["rhsm.log"]
    assert dev_listings["rhsm.log"]['se_type'] == 'rhsmcertd_log_t'


def test_ls_laZ():
    ls = LSlaZ(context_wrap(LS_LAZ_CONTENT1))
    assert '/var/log/anaconda' not in ls

    dev_listings = ls.listing_of('/var/log')
    assert "hawkey.log-20230924" in dev_listings
    assert dev_listings["hawkey.log-20230924"]['se_type'] == 'rpm_log_t'
    assert dev_listings["hawkey.log-20230924"]['size'] == 5220

    ls = LSlaZ(context_wrap(LS_LAZ_CONTENT2))
    assert '/var/log/rhsm' in ls

    dev_listings = ls.listing_of('/var/log/rhsm')
    assert "rhsm.log" in dev_listings
    assert 'se_type' in dev_listings["rhsm.log"]
    assert dev_listings["rhsm.log"]['se_type'] == 'rhsmcertd_log_t'

    dev_listings = ls.listing_of('/dev')
    assert 'stderr' in dev_listings
    assert dev_listings["stderr"]['link'] == '/proc/self/fd/2'

    ls = LSlaZ(context_wrap(LS_LAZ_CONTENT3_RHEL8_SELINUX))
    assert '/etc' in ls

    dev_listings = ls.listing_of('/etc')
    assert "bashrc" in dev_listings
    assert dev_listings["bashrc"]['se_user'] == '?'
    assert dev_listings["bashrc"]['se_role'] is None
    assert dev_listings["bashrc"]['se_type'] is None
    assert "chrony.conf" in dev_listings
    assert 'se_type' in dev_listings["chrony.conf"]
    assert dev_listings["chrony.keys"]['se_type'] == 'chronyd_keys_t'

    dev_listings = ls.listing_of('/var/log')
    assert 'boot.log-20220705' in dev_listings
    assert dev_listings["boot.log-20220705"]['size'] == 22027


def test_ls_laZ_on_dev():
    ls = LSlaRZ(context_wrap(LS_LARZ_DEV_CONTENT1))
    assert '/dev' in ls
    dev_listings = ls.listing_of('/dev')
    assert "autofs" in dev_listings
    assert 'se_type' in dev_listings["autofs"]
    assert dev_listings["autofs"]['se_type'] == 'autofs_device_t'
    assert dev_listings["autofs"]['major'] == 10
    assert dev_listings["autofs"]['minor'] == 235
    assert "block" in dev_listings
    assert dev_listings['block']['size'] == 140

    dev_listings = ls.listing_of('/dev/vfio')
    assert 'vfio' in dev_listings
    assert dev_listings["vfio"]['link'] == 'false_link_2'


def test_ls_ldH():
    ls = LSldH(context_wrap(LS_LDH))
    assert ls.error_lines == ["ls: cannot access '/dev/socketfile': No such file or directory"]
    assert ls.dirs == ['/boot', '/boot/grub2']
    assert '/boot/grub2/grub.cfg' in ls.files
    assert len(ls.files) == 1
    assert '/dev/vda' in ls.specials

    assert ls['entries']['/boot/grub2/grub.cfg']['type'] == '-'
    assert ls['entries']['/boot/grub2/grub.cfg']['perms'] == 'rw-------.'
    assert ls['entries']['/boot/grub2/grub.cfg']['links'] == 1
    assert ls['entries']['/boot/grub2/grub.cfg']['owner'] == 'root'
    assert ls['entries']['/boot/grub2/grub.cfg']['group'] == 'root'
    assert ls['entries']['/boot/grub2/grub.cfg']['size'] == 6658
    assert ls['entries']['/boot/grub2/grub.cfg']['name'] == '/boot/grub2/grub.cfg'
    assert ls['entries']['/boot/grub2/grub.cfg']['date'] == 'Dec 20  2023'

    assert ls.entries['/dev/vda']['type'] == 'b'
    assert ls.entries['/dev/vda']['owner'] == 'root'
    assert ls.entries['/dev/vda']['name'] == '/dev/vda'

    assert ls.raw_entry_of(
        '/dev/vda') == 'brw-rw----. 1 root disk 252, 0 Nov 16 03:19 /dev/vda'
    assert ls.raw_entry_of('/dev/vdb') is None
    assert ls.raw_entry_of(
        '/boot/grub2/grub.cfg') == '-rw-------. 1 root root 6658 Dec 20  2023 /boot/grub2/grub.cfg'
    assert ls.raw_entry_of(
        '/dev/stderr') == 'crw--w----. 1 root tty 136, 0 Nov 17 01:28 /dev/stderr'


def test_ls_ldZ():
    ls = LSldZ(context_wrap(LS_LDZ))
    assert ls.dirs == ['/boot']
    assert '/boot/grub2/grub.cfg' in ls.files
    assert len(ls.files) == 3
    assert '/dev/vda' in ls.specials

    assert ls['entries']['/boot/grub2/grub.cfg']['se_user'] == 'system_u'
    assert ls['entries']['/boot/grub2/grub.cfg']['se_role'] == 'object_r'
    assert ls['entries']['/boot/grub2/grub.cfg']['se_type'] == 'boot_t'
    assert ls['entries']['/boot/grub2/grub.cfg']['se_mls'] == 's0'

    assert ls.raw_entry_of('/dev/vda') == 'brw-rw----. 1 root disk system_u:object_r:fixed_disk_device_t:s0 252, 0 Nov  7 09:51 /dev/vda'
    assert ls.raw_entry_of('/dev/vdb') is None
    assert ls.raw_entry_of(
        '/boot/grub2/grub.cfg') == '-rw-------. 1 root root system_u:object_r:boot_t:s0 6658 Dec 20  2023 /boot/grub2/grub.cfg'
    assert ls.raw_entry_of(
        '/dev/stderr') == 'lrwxrwxrwx. 1 root root system_u:object_r:device_t:s0 15 Nov  6  2024 /dev/stderr -> /proc/self/fd/2'



# -*- coding: utf-8 -*-
"""
subscription-manager commands
=============================

Parsers for parsing output of the ``subscription-manager`` commands.

SubscriptionManagerID - command ``subscription-manager identity``
-----------------------------------------------------------------

SubscriptionManagerFacts - command ``subscription-manager facts``
-----------------------------------------------------------------

SubscriptionManagerStatus - command ``subscription-manager status``
-------------------------------------------------------------------

SubscriptionManagerSyspurpose - command ``subscription-manager syspurpose``
---------------------------------------------------------------------------
"""

import uuid

from insights.core import CommandParser, JSONParser
from insights.core.exceptions import SkipComponent
from insights.core.filters import add_filter
from insights.core.plugins import parser
from insights.specs import Specs

add_filter(
    Specs.subscription_manager_facts,
    [
        'conversions.activity',
        'image-builder.osbuild-composer.api-type',
        'instance_id',
    ],
)


def _local_kv_split(lines):
    ret = dict()
    for line in lines:
        # handle full-width colon
        line = line.replace('：', ': ')
        if ': ' in line:
            key, val = [_l.strip() for _l in line.split(': ', 1)]
            ret[key] = val
    if ret:
        return ret
    raise SkipComponent


@parser(Specs.subscription_manager_id)
class SubscriptionManagerID(CommandParser, dict):
    """
    Reads the output of subscription-manager identity and retrieves the UUID

    Example output::

        system identity: 6655c27c-f561-4c99-a23f-111111111111
        name: rhel7.localdomain
        org name: 1234567
        org ID: 1234567

    Examples::
        >>> type(subman_id)
        <class 'insights.parsers.subscription_manager.SubscriptionManagerID'>
        >>> subman_id.identity == '6655c27c-f561-4c99-a23f-111111111111'
        True
        >>> subman_id.get('org ID') == '1234567'
        True
        >>> subman_id.uuid == '6655c27c-f561-4c99-a23f-111111111111'
        True
    """

    def parse_content(self, content):
        self.update(_local_kv_split(content))

    @property
    def identity(self):
        """
        Returns the value of 'system identity'.
        Multiple language support is from:
        - https://github.com/candlepin/subscription-manager/tree/main/po
        """
        SYSID_MULT_LANG = [
            'system identity',  # en first
            'identidad de sistema',
            'identidade do sistema',
            'identità del sistema',
            'identité du système',
            'Systemidentität',
            'идентификация системы',
            'तंत्र पहचान',
            'प्रणाली ओळख',
            'চিস্টেম পৰিচয়',
            'সিস্টেম পরিচয়',
            'ਸਿਸਟਮ ਸ਼ਨਾਖਤ',
            'સિસ્ટમ ઓળખ',
            'ତନ୍ତ୍ର ପରିଚୟ',
            'கணினி அடையாளம்',
            'వ్యవస్థ గుర్తింపు',
            'ವ್ಯವಸ್ಥೆಯ ಗುರುತು',
            'സിസ്റ്റം ഐഡന്റിറ്റി',
            'სისტემის იდენტიფიკატორი',
            'システム ID',
            '系統身份',
            '系统身份',
            '시스템 ID',
        ]
        for name in SYSID_MULT_LANG:
            if name in self:
                return self[name]

    @property
    def uuid(self):
        """
        Returns the UUID of 'system identity' in standard format (32 digits separated by hyphens).
        """
        if self.identity:
            return str(uuid.UUID(self.identity))


@parser(Specs.subscription_manager_facts)
class SubscriptionManagerFacts(CommandParser, dict):
    """
    Class for parsing the output of `subscription-manager facts` command.

    Typical output of the command is::

        aws_instance_id: 567890567890
        network.ipv6_address: ::1
        uname.sysname: Linux
        uname.version: #1 SMP PREEMPT Fri Sep 2 16:07:40 EDT 2022
        virt.host_type: rhev, kvm
        virt.is_guest: True

    Examples:
        >>> type(rhsm_facts)
        <class 'insights.parsers.subscription_manager.SubscriptionManagerFacts'>
        >>> rhsm_facts['aws_instance_id']
        '567890567890'
    """

    def parse_content(self, content):
        self.update(_local_kv_split(content))


@parser(Specs.subscription_manager_status)
class SubscriptionManagerStatus(CommandParser, dict):
    """
    Reads the output of subscription-manager status

    Example output::

        +-------------------------------------------+
           System Status Details
        +-------------------------------------------+
        Overall Status: Disabled
        Content Access Mode is set to Simple Content Access. This host has access to content, regardless of subscription status.

        System Purpose Status: Disabled

    Examples::
        >>> type(subman_status)
        <class 'insights.parsers.subscription_manager.SubscriptionManagerStatus'>
        >>> subman_status['Overall Status'] == 'Disabled'
        True
        >>> subman_status['Content Access Mode'] == 'Simple Content Access'
        True
        >>> subman_status['System Purpose Status'] == 'Disabled'
        True
    """

    def parse_content(self, content):
        self.unparsed_lines = []
        for line in content:
            line = line.strip()

            if not line:
                continue

            if ': ' in line:
                key, val = [_l.strip() for _l in line.split(': ', 1)]
                self[key] = val
            elif line.startswith('Content Access Mode is set to'):
                self['Content Access Mode'] = (
                    line.split('.', 1)[0].split('Content Access Mode is set to')[1].strip()
                )
            elif line.startswith("Red Hat Enterprise Linux for Virtual Datacenters"):
                subscription_type = line.split(',')[1].strip(':').strip() if ',' in line else ''
                self['Red Hat Enterprise Linux for Virtual Datacenters'] = subscription_type
            elif "Guest has not been reported on any host" in line:
                self.unparsed_lines.append(line)

        if not self:
            raise SkipComponent


@parser(Specs.subscription_manager_syspurpose)
class SubscriptionManagerSyspurpose(CommandParser, JSONParser):
    """
    Reads the output of subscription-manager syspurpose

    Example output::

        {
          "addons": [],
          "role": "Red Hat Enterprise Linux Server",
          "service_level_agreement": "Standard",
          "usage": "Development/Test"
        }

    Examples::
        >>> type(subman_syspurpose)
        <class 'insights.parsers.subscription_manager.SubscriptionManagerSyspurpose'>
        >>> subman_syspurpose.get('role') == 'Red Hat Enterprise Linux Server'
        True
        >>> subman_syspurpose['service_level_agreement'] == 'Standard'
        True
        >>> subman_syspurpose['usage'] == 'Development/Test'
        True
    """

    pass



"""
Rules for data collection
"""

from __future__ import absolute_import

import json
import logging
import os
import six
import stat
import yaml

from six.moves import configparser as ConfigParser

from insights.client.constants import InsightsConstants as constants

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)
NETWORK = constants.custom_network_log_level


def correct_format(parsed_data, expected_keys, filename):
    '''
    Ensure the parsed file matches the needed format
    Returns True, <message> on error
    Returns False, None on success
    '''

    # validate keys are what we expect
    def is_list_of_strings(data):
        '''
        Helper function for correct_format()
        '''
        if data is None:
            # nonetype, no data to parse. treat as empty list
            return True
        if not isinstance(data, list):
            return False
        for l in data:
            if not isinstance(l, six.string_types):
                return False
        return True

    keys = parsed_data.keys()
    invalid_keys = set(keys).difference(expected_keys)
    if invalid_keys:
        return True, (
            'Unknown section(s) in %s: ' % filename
            + ', '.join(invalid_keys)
            + '\nValid sections are '
            + ', '.join(expected_keys)
            + '.'
        )

    # validate format (lists of strings)
    for k in expected_keys:
        if k in parsed_data:
            if k == 'patterns' and isinstance(parsed_data['patterns'], dict):
                if 'regex' not in parsed_data['patterns']:
                    return (
                        True,
                        'Patterns section contains an object but the "regex" key was not specified.',
                    )
                if 'regex' in parsed_data['patterns'] and len(parsed_data['patterns']) > 1:
                    return True, 'Unknown keys in the patterns section. Only "regex" is valid.'
                if not is_list_of_strings(parsed_data['patterns']['regex']):
                    return True, 'regex section under patterns must be a list of strings.'
                continue
            if not is_list_of_strings(parsed_data[k]):
                return True, '%s section must be a list of strings.' % k
    return False, None


def load_yaml(filename):
    try:
        with open(filename) as f:
            loaded_yaml = yaml.safe_load(f)
        if loaded_yaml is None:
            logger.debug('%s is empty.', filename)
            return {}
    except (yaml.YAMLError, yaml.parser.ParserError) as e:
        # can't parse yaml from conf
        raise RuntimeError(
            'ERROR: Cannot parse %s.\n'
            'If using any YAML tokens such as [] in an expression, '
            'be sure to wrap the expression in quotation marks.\n\nError details:\n%s\n'
            % (filename, e)
        )
    if not isinstance(loaded_yaml, dict):
        # loaded data should be a dict with at least one key
        raise RuntimeError('ERROR: Invalid YAML loaded.')
    return loaded_yaml


def verify_permissions(f):
    '''
    Verify 600 permissions on a file
    '''
    mode = stat.S_IMODE(os.stat(f).st_mode)
    if not mode == 0o600:
        raise RuntimeError("Invalid permissions on %s. " "Expected 0600 got %s" % (f, oct(mode)))
    logger.debug("Correct file permissions on %s", f)


class InsightsUploadConf(object):
    """
    Insights spec configuration
    """

    def __init__(self, config, conn=None):
        """
        Load configuration from parent
        """
        self.config = config
        self.remove_file = config.remove_file
        self.redaction_file = config.redaction_file
        self.content_redaction_file = config.content_redaction_file
        self.tags_file = config.tags_file

        # set rm_conf as a class attribute so we can observe it
        # in blacklist_report
        self.rm_conf = None

        # attribute to set when using file-redaction.yaml instead of
        #   remove.conf, for reporting purposes. True by default
        #   since new format is favored.
        self.using_new_format = True

    def get_rm_conf_old(self):
        """
        Get excluded files config from remove_file.
        """
        # Convert config object into dict
        self.using_new_format = False
        parsedconfig = ConfigParser.RawConfigParser()
        if not self.remove_file:
            # no filename defined, return nothing
            logger.debug('remove_file is undefined')
            return None
        if not os.path.isfile(self.remove_file):
            logger.debug(
                '%s not found. No data files, commands,'
                ' or patterns will be ignored, and no keyword obfuscation will occur.',
                self.remove_file,
            )
            return None
        try:
            verify_permissions(self.remove_file)
        except RuntimeError as e:
            if self.config.validate:
                # exit if permissions invalid and using --validate
                raise RuntimeError('ERROR: %s' % e)
            logger.warning('WARNING: %s', e)
        try:
            parsedconfig.read(self.remove_file)
            sections = parsedconfig.sections()

            if not sections:
                # file has no sections, skip it
                logger.debug('Remove.conf exists but no parameters have been defined.')
                return None

            if sections != ['remove']:
                raise RuntimeError(
                    'ERROR: invalid section(s) in remove.conf. Only "remove" is valid.'
                )

            expected_keys = ('commands', 'files', 'patterns', 'keywords')
            rm_conf = {'new_format': False}
            for item, value in parsedconfig.items('remove'):
                if item not in expected_keys:
                    raise RuntimeError(
                        'ERROR: Unknown key in remove.conf: '
                        + item
                        + '\nValid keys are '
                        + ', '.join(expected_keys)
                        + '.'
                    )
                if six.PY3:
                    rm_conf[item] = [
                        v.strip()
                        for v in value.strip().encode('utf-8').decode('unicode-escape').split(',')
                    ]
                else:
                    rm_conf[item] = [
                        v.strip() for v in value.strip().decode('string-escape').split(',')
                    ]
            self.rm_conf = rm_conf
        except ConfigParser.Error as e:
            # can't parse config file at all
            logger.debug(e)
            logger.debug(
                'To configure using YAML, please use file-redaction.yaml and file-content-redaction.yaml.'
            )
            raise RuntimeError(
                'ERROR: Cannot parse the remove.conf file.\n'
                'See %s for more information.' % self.config.logging_file
            )
        logger.warning(
            'WARNING: remove.conf is deprecated. Please use file-redaction.yaml and file-content-redaction.yaml. See https://access.redhat.com/articles/4511681 for details.'
        )
        return self.rm_conf

    def load_redaction_file(self, fname):
        '''
        Load the YAML-style file-redaction.yaml
            or file-content-redaction.yaml files
        '''
        if fname not in (self.redaction_file, self.content_redaction_file):
            # invalid function use, should never get here in a production situation
            return None
        if not fname:
            # no filename defined, return nothing
            logger.debug('redaction_file or content_redaction_file is undefined')
            return None
        if not fname or not os.path.isfile(fname):
            if fname == self.redaction_file:
                logger.debug(
                    '%s not found. No files or commands will be skipped.', self.redaction_file
                )
            elif fname == self.content_redaction_file:
                logger.debug(
                    '%s not found. '
                    'No patterns will be skipped and no keyword obfuscation will occur.',
                    self.content_redaction_file,
                )
            return None
        try:
            verify_permissions(fname)
        except RuntimeError as e:
            if self.config.validate:
                # exit if permissions invalid and using --validate
                raise RuntimeError('ERROR: %s' % e)
            logger.warning('WARNING: %s', e)
        loaded = load_yaml(fname)
        if fname == self.redaction_file:
            err, msg = correct_format(loaded, ('commands', 'files', 'components'), fname)
        elif fname == self.content_redaction_file:
            err, msg = correct_format(loaded, ('patterns', 'keywords'), fname)
        if err:
            # YAML is correct but doesn't match the format we need
            raise RuntimeError('ERROR: ' + msg)
        return loaded

    def get_rm_conf(self):
        '''
        Try to load the file-redaction.yaml and file-content-redaction.yaml
        '''

        rm_conf = {}
        redact_conf = self.load_redaction_file(self.redaction_file)
        content_redact_conf = self.load_redaction_file(self.content_redaction_file)

        if not redact_conf and not content_redact_conf:
            # no file-redaction.yaml or file-content-redaction.yaml defined,
            #   try to use remove.conf
            self.rm_conf = self.get_rm_conf_old()
        else:
            rm_conf.update(new_format=True)
            rm_conf.update(redact_conf or {})
            rm_conf.update(content_redact_conf or {})
            # remove Nones, empty strings, and empty lists
            self.rm_conf = dict((k, v) for k, v in rm_conf.items() if v)

        if self.rm_conf:
            # WARNING for missing HBI required specs
            required = {
                'machine_id': ('files', '/etc/insights-client/machine-id'),
                'subscription_manager_id': ('commands', '/usr/sbin/subscription-manager identity'),
            }
            _prefix = 'insights.specs.default.DefaultSpecs.'
            for spec, (kind, cp) in required.items():
                if (
                    # symbolic name / spec name
                    any(spec in self.rm_conf.get(_k, []) for _k in ['files', 'commands'])
                    # file path / command line
                    or cp in self.rm_conf.get(kind, [])
                    # insights component module
                    or _prefix + spec in self.rm_conf.get('components', [])
                ):
                    logger.warning(
                        "WARNING: Spec %s will be skipped for redaction; as it would cause issues, please remove it from %s.",
                        spec,
                        self.redaction_file,
                    )
        # return the RAW rm_conf
        return self.rm_conf

    def get_tags_conf(self):
        '''
        Try to load the tags.conf file
        '''
        if not os.path.isfile(self.tags_file):
            logger.info("%s does not exist", self.tags_file)
            return None
        else:
            try:
                load_yaml(self.tags_file)
                logger.info("%s loaded successfully", self.tags_file)
            except RuntimeError:
                logger.warning("Invalid YAML. Unable to load %s", self.tags_file)
                return None

    def validate(self):
        '''
        Validate remove.conf and tags.conf
        '''
        self.get_tags_conf()
        success = self.get_rm_conf()
        if not success:
            logger.info('No contents in the blacklist configuration to validate.')
            return None
        # Using print here as this could contain sensitive information
        print('Blacklist configuration parsed contents:')
        print(json.dumps(success, indent=4))
        logger.info('Parsed successfully.')
        return True



from insights.parsers.keystone_log import KeystoneLog
from insights.tests import context_wrap

from datetime import datetime

KEYSTONE_LOG = """
2016-11-09 14:31:48.681 1082 WARNING oslo_config.cfg [-] Option "rabbit_userid" from group "oslo_messaging_rabbit" is deprecated for removal.  Its value may be silently ignored in the future.
2016-11-09 14:31:48.681 1082 WARNING oslo_config.cfg [-] Option "rabbit_password" from group "oslo_messaging_rabbit" is deprecated for removal.  Its value may be silently ignored in the future.
2016-11-09 14:31:48.774 1082 INFO keystone.cmd.cli [-] Created domain default
2016-11-09 14:31:48.802 1082 INFO keystone.cmd.cli [req-ace08b7c-d0d2-4b18-b792-1ec3402575b1 - - - - -] Created project admin
"""


def test_keystone_log():
    log = KeystoneLog(context_wrap(KEYSTONE_LOG))
    assert len(log.get('INFO')) == 2
    assert len(list(log.get_after(datetime(2016, 11, 9, 14, 31, 0)))) == 4



from insights.combiners.crio_conf import AllCrioConf
from insights.parsers.crio_conf import CrioConf
from insights.tests import context_wrap


CRIO_CONFIG = """
[crio]
storage_driver = "overlay"
storage_option = [
        "overlay.override_kernel_check=1",
]

[crio.runtime]
selinux = true

[crio.network]
plugin_dirs = [
        "/usr/libexec/cni",
]
[crio.metrics]
""".strip()

CRIO_CONMON_00_CONF = """
[crio]
internal_wipe = true
storage_driver = "device mapper"
""".strip()

CRIO_CONMON_99_CONF = """
[crio]
storage_driver = "overlay2"

[crio.api]
stream_address = ""
stream_port = "10010"

[crio.runtime]
selinux = true
conmon = ""
conmon_cgroup = "pod"
default_env = [
    "NSS_SDB_USE_CACHE=no",
]
log_level = "info"
cgroup_manager = "systemd"
""".strip()


def test_active_crio_conf_nest():
    crio_conf1 = CrioConf(context_wrap(CRIO_CONFIG, path='/etc/crio/crio.conf'))
    crio_conf2 = CrioConf(context_wrap(CRIO_CONMON_00_CONF, path='/etc/crio/crio.conf.d/00-conmon.conf'))
    crio_conf3 = CrioConf(context_wrap(CRIO_CONMON_99_CONF, path='/etc/crio/crio.conf.d/99-conmon.conf'))
    result = AllCrioConf([crio_conf1, crio_conf2, crio_conf3])
    assert result.has_option("crio", "internal_wipe")
    assert result.has_option("crio.runtime", "default_env")
    assert result.get("crio.api", "stream_port") == "\"10010\""
    assert result.get('crio', 'storage_driver') == "\"overlay2\""
    assert "crio.network" in result.sections()
    assert "realmstest" not in result.sections()
    assert result.has_section("crio.metrics")
    assert not result.has_option("crio.metrics", "nosuchoption")
    assert not result.options("realmsno")
    assert result.files == ['/etc/crio/crio.conf', '/etc/crio/crio.conf.d/00-conmon.conf',
                            '/etc/crio/crio.conf.d/99-conmon.conf']



import os
import json
import pytest

from collections import defaultdict
from mock.mock import patch

from insights import collect
from insights.cleaner import Cleaner
from insights.client.archive import InsightsArchive
from insights.client.config import InsightsConfig
from insights.core import dr, filters
from insights.core.exceptions import ContentException
from insights.core.context import HostContext
from insights.core.filters import add_filter
from insights.core.plugins import datasource
from insights.core.spec_factory import (
    RegistryPoint,
    SpecSet,
    command_with_args,
    foreach_collect,
    foreach_execute,
    glob_file,
    simple_command,
    simple_file,
    first_file,
    first_of,
)

here = os.path.abspath(os.path.dirname(__file__))

FILTER_DATA = "Some test data"

this_file = os.path.abspath(__file__).rstrip("c")
test_large_file = '/tmp/_insights_test.large_file_filter'
test_large_file_without_filter = '/tmp/_insights_test.large_file_without_filter'
test_one_line_left = '/tmp/_insights_test.one_line_left'
test_file_empty_after_filter = '/tmp/_insights_test.file_empty_after_filter'

specs_manifest = """
---
version: 0

client:
    context:
        class: insights.core.context.HostContext
        args:
            timeout: 5

    # commands and files to ignore
    blacklist:
        files: []
        commands: []
        patterns: ['TeSt_']
        keywords: []

    persist:
        - name: insights.tests.specs.test_specs_filters.Specs
          enabled: true

    run_strategy:
        name: serial
        args:
            max_workers: null

plugins:
    default_component_enabled: false

    packages:
        - insights.tests.specs.test_specs_filters

    configs:
        - name: insights.core.spec_factory
          enabled: true
        - name: insights.tests.specs.test_specs_filters.Specs
          enabled: true
        - name: insights.tests.specs.test_specs_filters.Stuff
          enabled: true
        - name: insights.tests.specs.test_specs_filters.dostuff
          enabled: true
""".strip()


class Specs(SpecSet):
    many_glob_filter = RegistryPoint(multi_output=True, no_redact=True, filterable=True)
    many_foreach_exe_filter = RegistryPoint(multi_output=True, filterable=True)
    many_foreach_clc_filter = RegistryPoint(multi_output=True, filterable=True)
    smpl_cmd_w_filter = RegistryPoint(filterable=True)
    smpl_file_w_filter = RegistryPoint(filterable=True)
    first_file_spec_w_filter = RegistryPoint(filterable=True)
    first_of_spec_w_filter = RegistryPoint(filterable=True)
    file_empty_after_filter = RegistryPoint(filterable=True)
    cmd_empty_after_filter = RegistryPoint(filterable=True)
    cmd_w_args_filter = RegistryPoint(filterable=True)
    large_filter = RegistryPoint(filterable=True)
    large_file_without_filter = RegistryPoint(filterable=False)
    one_line_left = RegistryPoint(filterable=True)


class Stuff(Specs):
    many_glob_filter = glob_file(here + "/../test_b*.py")

    @datasource(HostContext)
    def files1(broker):
        """Return a list of directories from the spec filter"""
        return [here + "/../test_test.py", here + "/../test_taglang.py"]

    @datasource(HostContext)
    def files2(broker):
        """Return a list of directories from the spec filter"""
        return ' '.join([here + "/../test_test.py", here + "/../test_taglang.py"])

    many_foreach_exe_filter = foreach_execute(files1, 'ls -l %s')
    many_foreach_clc_filter = foreach_collect(files1, "%s", no_redact=True)
    smpl_cmd_w_filter = simple_command("echo -n ' hello 1'")
    smpl_file_w_filter = simple_file(here + "/../mock_web_server.py")
    first_file_spec_w_filter = first_file([here + "/../spec_tests.py", "/etc/os-release"])
    first_of_spec_w_filter = first_of(
        [
            simple_command("echo -n ' hello 2'"),
            simple_file(this_file),
        ]
    )

    file_empty_after_filter = simple_file(test_file_empty_after_filter)
    cmd_empty_after_filter = simple_command("echo -n ' no_filter'")
    cmd_w_args_filter = command_with_args('ls -lt %s', files2)
    large_filter = simple_file(test_large_file)
    large_file_without_filter = simple_file(test_large_file_without_filter)
    one_line_left = simple_file(test_one_line_left)


class stage(dr.ComponentType):
    def invoke(self, broker):
        return self.component(broker)


@stage(
    Stuff.many_glob_filter,
    Stuff.many_foreach_exe_filter,
    Stuff.many_foreach_clc_filter,
    Stuff.smpl_cmd_w_filter,
    Stuff.smpl_file_w_filter,
    Stuff.first_file_spec_w_filter,
    Stuff.first_of_spec_w_filter,
    Stuff.cmd_w_args_filter,
    Stuff.large_filter,
    Stuff.large_file_without_filter,
    Stuff.one_line_left,
    Stuff.file_empty_after_filter,
    Stuff.cmd_empty_after_filter,
)
def dostuff(broker):
    assert Stuff.many_glob_filter in broker
    assert Stuff.many_foreach_exe_filter in broker
    assert Stuff.many_foreach_clc_filter in broker
    assert Stuff.smpl_cmd_w_filter in broker
    assert Stuff.smpl_file_w_filter in broker
    assert Stuff.first_file_spec_w_filter in broker
    assert Stuff.first_of_spec_w_filter in broker
    assert Stuff.cmd_w_args_filter in broker
    assert Stuff.one_line_left in broker
    assert Stuff.file_empty_after_filter in broker
    assert Stuff.cmd_empty_after_filter in broker


# File content
with open(here + "/../mock_web_server.py") as f:
    smpl_file_w_filter_content = [l for l in f.read().splitlines() if "def get" in l]
with open(here + "/../spec_tests.py") as f:
    first_file_w_filter_content = [
        l for l in f.read().splitlines() if any(i in l for i in ["def report_", "rhel"])
    ]

#
# TEST
#


def setup_function(func):
    if func in (test_specs_filters_collect, test_specs_filters_spec_factory):
        # empty relevant files
        with open(test_file_empty_after_filter, 'w') as t:
            t.write('no-filter')
    with open(test_large_file, 'w') as fd:
        for i in range(filters.MAX_MATCH + 1):
            fd.write(str(i) + FILTER_DATA + '\n')
    with open(test_large_file_without_filter, 'w') as fd:
        for i in range(filters.MAX_MATCH + 1):
            fd.write(str(i) + FILTER_DATA + '\n')
    with open(test_one_line_left, 'w') as fd:
        for i in range(filters.MAX_MATCH + 1):
            fd.write(str(i) + FILTER_DATA + '\n')


def teardown_function(func):
    # Reset Test ENV
    filters._CACHE = {}
    filters.FILTERS = defaultdict(dict)
    dr.COMPONENTS = defaultdict(lambda: defaultdict(set))
    dr.TYPE_OBSERVERS = defaultdict(set)
    dr.ENABLED = defaultdict(lambda: True)

    if os.path.exists(test_file_empty_after_filter):
        os.remove(test_file_empty_after_filter)
    if os.path.exists(test_large_file):
        os.remove(test_large_file)
    if os.path.exists(test_large_file_without_filter):
        os.remove(test_large_file_without_filter)
    if os.path.exists(test_one_line_left):
        os.remove(test_one_line_left)


def test_specs_filters_spec_factory():
    add_filter(Stuff.many_glob_filter, " ")
    add_filter(Stuff.many_foreach_exe_filter, " ")
    add_filter(Stuff.many_foreach_clc_filter, " ")
    add_filter(Stuff.smpl_cmd_w_filter, " hello ")
    add_filter(Stuff.smpl_file_w_filter, "def get")
    add_filter(Stuff.first_file_spec_w_filter, ["def report_", "rhel"])
    add_filter(Stuff.first_of_spec_w_filter, ["def test", " hello "])
    add_filter(Stuff.file_empty_after_filter, " hello ")
    add_filter(Stuff.cmd_empty_after_filter, " hello ")
    add_filter(Stuff.cmd_w_args_filter, [" ", ":"])
    add_filter(Stuff.large_filter, ["Some"])
    add_filter(Stuff.one_line_left, ["test data"], 1)
    broker = dr.Broker()
    broker[HostContext] = HostContext()
    broker['cleaner'] = Cleaner(None, None)
    broker = dr.run(dr.get_dependency_graph(dostuff), broker)

    assert dostuff in broker, broker.tracebacks
    # "filter" works when loading
    assert "hello" in broker[Stuff.smpl_cmd_w_filter].content[0]
    assert len(broker[Stuff.smpl_cmd_w_filter].content) == 1
    assert broker[Stuff.smpl_file_w_filter].content == smpl_file_w_filter_content
    assert broker[Stuff.first_file_spec_w_filter].content == first_file_w_filter_content
    assert len(broker[Stuff.first_of_spec_w_filter].content) == 1
    with pytest.raises(ContentException) as ce:
        assert broker[Stuff.file_empty_after_filter].content is None
    assert "Empty (after filtering)" in str(ce)
    assert "file_empty_after_filter" in str(ce)
    with pytest.raises(ContentException) as ce:
        assert broker[Stuff.cmd_empty_after_filter].content is None
    assert "Empty (after filtering)" in str(ce)
    assert "no_filter" in str(ce)


def test_line_terminators():
    add_filter(Stuff.many_glob_filter, " ")
    add_filter(Stuff.many_foreach_exe_filter, " ")
    add_filter(Stuff.many_foreach_clc_filter, " ")
    add_filter(Stuff.smpl_cmd_w_filter, " hello ")
    add_filter(Stuff.smpl_file_w_filter, "def get")
    add_filter(Stuff.first_file_spec_w_filter, ["def report_", "rhel"])
    add_filter(Stuff.first_of_spec_w_filter, ["def test", " hello "])
    add_filter(Stuff.file_empty_after_filter, " hello ")
    add_filter(Stuff.cmd_empty_after_filter, " hello ")
    add_filter(Stuff.cmd_w_args_filter, [" ", ":"])
    add_filter(Stuff.large_filter, ["Some"])
    add_filter(Stuff.one_line_left, ["test data"], 1)
    broker = dr.Broker()
    broker[HostContext] = HostContext()
    broker['cleaner'] = Cleaner(None, None)
    broker = dr.run(dr.get_dependency_graph(dostuff), broker)

    content = broker[Stuff.smpl_file_w_filter].content
    assert not any(l.endswith("\n") for l in content)
    # "filter" works only when writing
    # assert all("def get" in l for l in content), content
    assert "hello" in broker[Stuff.smpl_cmd_w_filter].content[0]
    assert len(broker[Stuff.smpl_cmd_w_filter].content) == 1
    assert len(broker[Stuff.smpl_file_w_filter].content) == len(smpl_file_w_filter_content)
    assert len(broker[Stuff.first_file_spec_w_filter].content) == len(first_file_w_filter_content)
    assert len(broker[Stuff.first_of_spec_w_filter].content) == 1


def test_exp_no_filters():
    broker = dr.Broker()
    broker[HostContext] = HostContext()
    broker['cleaner'] = Cleaner(None, None)
    broker = dr.run(dr.get_dependency_graph(dostuff), broker)
    assert dostuff not in broker
    exception_cnt = 0
    for spec, info in broker.exceptions.items():
        # warning of lacking filters
        if any("due to" in str(msg) for msg in info):
            if "smpl_file_w_filter" in str(spec):
                exception_cnt += 1
            elif "smpl_cmd_w_filter" in str(spec):
                exception_cnt += 10
            elif "first_file_spec_w_filter" in str(spec):
                exception_cnt += 100
            elif "first_of_spec_w_filter" in str(spec):
                exception_cnt += 1000
            elif "many_glob_filter" in str(spec):
                exception_cnt += 10000
            elif "many_foreach_exe_filter" in str(spec):
                exception_cnt += 100000
            elif "many_foreach_clc_filter" in str(spec):
                exception_cnt += 1000000
            elif "cmd_w_args_filter" in str(spec):
                exception_cnt += 10000000
            elif "large_filter" in str(spec):
                exception_cnt += 100000000
            elif "one_line_left" in str(spec):
                exception_cnt += 1000000000
    assert exception_cnt == 1111111111


@pytest.mark.parametrize("obfuscation_list", ['hostname', 'ipv4', 'ipv6', 'mac'], [])
@patch('insights.cleaner.Cleaner.generate_report', return_value=None)
def test_specs_filters_collect(gen, obfuscation_list):
    add_filter(Stuff.many_glob_filter, " ")
    add_filter(Stuff.many_foreach_exe_filter, " ")
    add_filter(Stuff.many_foreach_clc_filter, " ")
    add_filter(Stuff.smpl_cmd_w_filter, " hello ")
    add_filter(Stuff.smpl_file_w_filter, "def get")
    add_filter(Stuff.first_file_spec_w_filter, [" hello ", "Test"])
    add_filter(Stuff.first_of_spec_w_filter, ["def test", " hello "])
    add_filter(Stuff.file_empty_after_filter, " hello ")
    add_filter(Stuff.cmd_empty_after_filter, " hello ")
    add_filter(Stuff.cmd_w_args_filter, [" ", ":"])
    add_filter(Stuff.large_filter, ["Some"])
    add_filter(Stuff.one_line_left, ["test data"], 1)
    # Preparation
    manifest = collect.load_manifest(specs_manifest)
    for pkg in manifest.get("plugins", {}).get("packages", []):
        dr.load_components(pkg, exclude=None)
    conf = InsightsConfig(obfuscation_list=obfuscation_list, manifest=manifest)
    arch = InsightsArchive(conf)
    arch.create_archive_dir()
    output_path, errors = collect.collect(
        tmp_path=arch.tmp_dir, archive_name=arch.archive_name, client_config=conf
    )
    meta_data_root = os.path.join(output_path, 'meta_data')
    data_root = os.path.join(output_path, 'data')

    assert not errors
    count = 0
    for spec in Specs.__dict__:
        if str(spec) == "large_file_without_filter":
            # skip the  without filter file
            continue
        if not spec.startswith(('__', 'context_handlers', 'registry')):
            file_name = "insights.tests.specs.test_specs_filters.Specs.{0}.json".format(spec)
            meta_data = os.path.join(meta_data_root, file_name)
            with open(meta_data, 'r') as fp:
                mdata = json.load(fp)
                errors = mdata.get('errors')
                results = mdata.get('results')
                for error in errors:
                    if "empty_after_filter" in spec:
                        assert "Empty (after filtering)" in error
                if not isinstance(results, list):
                    results = [results]
                count += 1
                for result in results:
                    if not result:
                        continue
                    rel = result['object']['relative_path']
                    org_content = new_content = None
                    with open(os.path.join(data_root, rel), 'r') as fp:
                        new_content = fp.readlines()
                    if "insights_commands" not in rel:
                        # check files only
                        with open("/" + rel, 'r') as fp:
                            org_content = fp.readlines()
                        assert len(org_content) > len(new_content)
                        if "large_filter" in spec:
                            # if matched lines exceed the MAX_MATCH
                            # collect the last MAX_MATCH lines only
                            assert len(new_content) == filters.MAX_MATCH
                            assert new_content[0] != org_content[0]
                            assert new_content[-1].strip() == org_content[-1].strip()
                        elif "one_line_left" in spec:
                            # only one line is required
                            assert len(new_content) == 1
                            assert new_content[0] != org_content[0]
                            # the last line is kept
                            assert new_content[-1].strip() == org_content[-1].strip()
    assert count == 12  # Number of Specs with results

    arch.delete_archive_dir()



"""
Custom datasources for logrotate
"""

import os

from insights.core.context import HostContext
from insights.core.plugins import datasource


@datasource(HostContext)
def logrotate_conf_list(broker):
    """
    This datasource returns the list of logrotate configuration files.
    - "/etc/logrotate.conf" is the one must be collected.
    - Files without extension under "/etc/logrotate.d/" will be collected.
    - Files with extension, only the ".conf" files will be collected.

    Returns:
        list: The list of logrotate configuration files.
              ["/etc/logrotate.conf"] by default
    """
    conf_files = ["/etc/logrotate.conf"]
    root = "/etc/logrotate.d/"
    if os.path.isdir(root):
        for file_name in os.listdir(root):
            file_path = os.path.join(root, file_name)
            if ('.' not in file_name or file_name.endswith('.conf')) and os.path.isfile(file_path):
                conf_files.append(file_path)
    return conf_files



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import kpatch_list
from insights.tests import context_wrap

NORMAL_OUTPUT = '''
Loaded patch modules:
kpatch_3_10_0_1062_1_1_1_4 [enabled]

Installed patch modules:
kpatch_3_10_0_1062_1_1_1_4 (3.10.0-1062.1.1.el7.x86_64)
'''.strip()

MULTILINE_OUTPUT = '''
Loaded patch modules:
kpatch_3_10_0_1062_1_1_1_4 [enabled]
kpatch_3_10_0_1062_1_1_1_5 [enabled]

Installed patch modules:
kpatch_3_10_0_1062_1_1_1_4 (3.10.0-1062.1.1.el7.x86_64)
kpatch_3_10_0_1062_1_1_1_5 (3.10.0-1062.1.2.el7.x86_64)
'''.strip()

BAD_OUTPUT1 = ''

BAD_OUTPUT2 = '''
Loaded patch modules:
kpatch_3_10_0_1062_1_1_1_4 [enabled]
kpatch_3_10_0_1062_1_1_1_5

Installed patch modules:
kpatch_3_10_0_1062_1_1_1_4 (3.10.0-1062.1.1.el7.x86_64)
(3.10.0-1062.1.2.el7.x86_64)
'''.strip()


def test_doc_examples():
    env = {
        'kpatchs': kpatch_list.KpatchList(
            context_wrap(NORMAL_OUTPUT)),
    }
    failed, total = doctest.testmod(kpatch_list, globs=env)
    assert failed == 0


def test_kpatch_list():
    kpatchs = kpatch_list.KpatchList(context_wrap(MULTILINE_OUTPUT))
    assert len(kpatchs.loaded) > 0
    assert len(kpatchs.installed) > 0
    assert kpatchs.loaded.get('kpatch_3_10_0_1062_1_1_1_4') == "enabled"
    assert kpatchs.installed.get('kpatch_3_10_0_1062_1_1_1_4') == "3.10.0-1062.1.1.el7.x86_64"
    assert kpatchs.loaded.get('kpatch_3_10_0_1062_1_1_1_5') == "enabled"
    assert kpatchs.installed.get('kpatch_3_10_0_1062_1_1_1_5') == "3.10.0-1062.1.2.el7.x86_64"

    kpatchs = kpatch_list.KpatchList(context_wrap(BAD_OUTPUT2))
    assert len(kpatchs.loaded) == 2
    assert len(kpatchs.installed) == 1
    assert kpatchs.loaded.get('kpatch_3_10_0_1062_1_1_1_4') == "enabled"
    assert kpatchs.installed.get('kpatch_3_10_0_1062_1_1_1_4') == "3.10.0-1062.1.1.el7.x86_64"
    assert kpatchs.loaded.get('kpatch_3_10_0_1062_1_1_1_5') == ''


def test_fail():
    with pytest.raises(SkipComponent) as e:
        kpatch_list.KpatchList(context_wrap(BAD_OUTPUT1))
    assert "No Data from command: /usr/sbin/kpatch list" in str(e)



import datetime
import pytest

from insights.core.exceptions import ParseException
from insights.parsers.uptime import Uptime
from insights.tests import context_wrap

UPTIME_TEST_DATA = [
    {
        'test_data': " 14:28:24 up  5:55,  4 users,  load average: 0.04, 0.03, 0.05",
        'currtime': '14:28:24', 'updays': '', 'uphhmm': '5:55', 'users': '4',
        'loadavg': ['0.04', '0.03', '0.05'],
        'uptime': datetime.timedelta(days=0, hours=5, minutes=55)
    }, {
        'test_data': " 10:55:22 up 40 days, 21:17,  1 user,  load average: 0.49, 0.12, 0.04",
        'currtime': '10:55:22', 'updays': '40', 'uphhmm': '21:17', 'users': '1',
        'loadavg': ['0.49', '0.12', '0.04'],
        'uptime': datetime.timedelta(days=40, hours=21, minutes=17)
    }, {
        'test_data': " 10:55:22 up 40 days, 3 min,  1 user,  load average: 0.49, 0.12, 0.04",
        'currtime': '10:55:22', 'updays': '40', 'uphhmm': '00:03', 'users': '1',
        'loadavg': ['0.49', '0.12', '0.04'],
        'uptime': datetime.timedelta(days=40, hours=0, minutes=3)
    }, {
        'test_data': " 10:55:22 up 30 min,  1 user,  load average: 0.49, 0.12, 0.04",
        'currtime': '10:55:22', 'updays': '', 'uphhmm': '00:30', 'users': '1',
        'loadavg': ['0.49', '0.12', '0.04'],
        'uptime': datetime.timedelta(days=0, hours=0, minutes=30)
    }, {
        'test_data': " 16:33:40 up 2 days, 12 users,  load average: 9.32, 8.96, 8.87",
        'currtime': '16:33:40', 'updays': '2', 'uphhmm': '', 'users': '12',
        'loadavg': ['9.32', '8.96', '8.87'],
        'uptime': datetime.timedelta(days=2, hours=0, minutes=0)
    }
]
NOT_AN_UPTIME = "10:55:22 up 30 min"


def total_seconds(time_delta):
    return (time_delta.days * 24 * 60 * 60) + time_delta.seconds


def test_get_uptimes():
    for test_data in UPTIME_TEST_DATA:
        uptime = Uptime(context_wrap(test_data['test_data']))
        assert len(uptime.data) == 6
        assert uptime.currtime == test_data['currtime']
        assert uptime.updays == test_data['updays']
        assert uptime.uphhmm == test_data['uphhmm']
        assert uptime.users == test_data['users']
        assert uptime.loadavg == test_data['loadavg']
        assert total_seconds(uptime.uptime) == total_seconds(test_data['uptime'])


def test_get_no_uptime():
    with pytest.raises(ParseException) as exc:
        uptime = Uptime(context_wrap(NOT_AN_UPTIME))
        assert len(uptime.data) == 0
    assert 'No uptime data found on ' in str(exc)



import os
from setuptools import setup, find_packages

__here__ = os.path.dirname(os.path.abspath(__file__))

package_info = dict.fromkeys(["RELEASE", "COMMIT", "VERSION", "NAME"])

for name in package_info:
    with open(os.path.join(__here__, "insights", name)) as f:
        package_info[name] = f.read().strip()

entry_points = {
    'console_scripts': [
        'insights-collect = insights.collect:main',
        'insights-run = insights:main',
        'insights = insights.command_parser:main',
        'insights-cat = insights.tools.cat:main',
        'insights-dupkeycheck = insights.tools.dupkeycheck:main',
        'insights-inspect = insights.tools.insights_inspect:main',
        'insights-info = insights.tools.query:main',
        'mangle = insights.util.mangle:main',
    ]
}

runtime = set(
    [
        'six',
        'requests',
        'redis',
        'rpm; python_version >= "3.6"',
        'cachecontrol',
        'cachecontrol[redis]',
        'cachecontrol[filecache]',
        'defusedxml',
        'lockfile',
        'jinja2<=2.11.3; python_version <= "2.7"',
        'jinja2; python_version > "2.7" and python_version <= "3.6"',
        'jinja2>=3.1.6;  python_version > "3.6"',
        'pyyaml',
        'setuptools; python_version >= "3.12"',
    ]
)


def maybe_require(pkg):
    try:
        __import__(pkg)
    except ImportError:
        runtime.add(pkg)


maybe_require("importlib")
maybe_require("argparse")


client = set(
    [
        'requests',
        'python-gnupg==0.4.6',
        'oyaml',
    ]
)

develop = set(
    [
        'pre-commit',
        'wheel',
    ]
)

docs = set(
    [
        'docutils',
        'Sphinx<=8.2.3',
        'nbsphinx',
        'sphinx_rtd_theme',
        'ipython<8.7.0',
        'MarkupSafe==2.0.1',
        'colorama',
        'Pygments',
        'jedi',
    ]
)

testing = set(
    [
        'coverage',
        'pytest~=4.6.0; python_version == "2.7"',
        'pytest; python_version >= "3"',
        'pytest-cov',
        'mock==2.0.0',
    ]
)

cluster = set(
    [
        'ansible',
        'pandas',
        'colorama',
    ]
)

openshift = set(['openshift'])

linting = set(['flake8'])

optional = set(
    [
        'python-cjson',
        'python-logstash',
        'python-statsd',
        'watchdog',
    ]
)

if __name__ == "__main__":
    # allows for runtime modification of rpm name
    name = os.environ.get("INSIGHTS_CORE_NAME", package_info["NAME"])

    setup(
        name=name,
        version=package_info["VERSION"],
        description="Insights Core is a data collection and analysis framework",
        long_description=open(os.path.join(__here__, "README.rst")).read(),
        url="https://github.com/redhatinsights/insights-core",
        author="Red Hat, Inc.",
        author_email="insights@redhat.com",
        packages=find_packages(),
        install_requires=list(runtime),
        package_data={'': ['LICENSE']},
        license='Apache 2.0',
        extras_require={
            'develop': list(runtime | develop | client | docs | linting | testing | cluster),
            'client': list(runtime | client),
            'client-develop': list(runtime | develop | client | linting | testing),
            'cluster': list(runtime | cluster),
            'openshift': list(runtime | openshift),
            'optional': list(optional),
            'docs': list(docs),
            'linting': list(linting | client),
            'testing': list(testing | client),
        },
        classifiers=[
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'Natural Language :: English',
            'License :: OSI Approved :: Apache Software License',
            'Programming Language :: Python',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3.3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: Python :: 3.11',
        ],
        entry_points=entry_points,
        include_package_data=True,
    )



"""
MysqlLog - File ``/var/log/mysqld.log``
=======================================

Module for parsing the log file for Mysql, including ``/var/log/mysql/mysqld.log``,
``/var/log/mysqld.log`` and ``/var/opt/rh/rh-mysql*/log/mysql/mysqld.log``.

.. note::
    By default, logrotate of mysql.log is set to daily.
    In some lines of log, there are no timestamp.

Typical content of ``mysql.log`` file is::

    2018-03-13T06:37:39.651387Z 0 [Warning] InnoDB: New log files created, LSN=45790
    2018-03-13T06:37:39.719166Z 0 [Warning] InnoDB: Creating foreign key constraint system tables.
    2018-03-13T06:37:39.784406Z 0 [Warning] No existing UUID has been found, so we assume that this is the first time that this server has been started. Generating a new UUID: 0
    698a7d6-2689-11e8-8944-0800274ac5ef.
    2018-03-13T06:37:39.789636Z 0 [Warning] Gtid table is not ready to be used. Table 'mysql.gtid_executed' cannot be opened.
    2018-03-13T06:37:40.498084Z 0 [Warning] CA certificate ca.pem is self signed.
    2018-03-13T06:37:41.080591Z 1 [Warning] root@localhost is created with an empty password ! Please consider switching off the --initialize-insecure option.
    md5_dgst.c(80): OpenSSL internal error, assertion failed: Digest MD5 forbidden in FIPS mode!
    06:37:41 UTC - mysqld got signal 6 ;
    2018-03-13T07:43:31.450772Z 0 [Note] Event Scheduler: Loaded 0 events
    2018-03-13T07:43:31.450988Z 0 [Note] /opt/rh/rh-mysql57/root/usr/libexec/mysqld: ready for connections.
    Version: '5.7.16'  socket: '/var/lib/mysql/mysql.sock'  port: 3306  MySQL Community Server (GPL)

Examples:

    >>> my = MysqlLog((context_wrap(MYSQLLOG)))
    >>> my.get('mysqld')[0]['raw_message']
    '06:37:41 UTC - mysqld got signal 6 ;'
    >>> 'ready for connections' in my
    True
"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.mysql_log)
class MysqlLog(LogFileOutput):
    """Class for parsing ``/var/log/mysqld.log`` file.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`
    """
    pass



from insights.parsr.examples.arith import evaluate


def test_single_ops():
    assert evaluate("2+3") == 5
    assert evaluate("1-3") == -2
    assert evaluate("2*3") == 6
    assert evaluate("4/2") == 2


def test_multiple_ops():
    assert evaluate("2+3+4") == 9
    assert evaluate("1-3+2") == 0
    assert evaluate("2*3*4") == 24
    assert evaluate("3*4/2") == 6


def test_nested():
    assert evaluate("24-2*(3+4)") == 10


def test_spaces():
    assert evaluate("24 - 2 * ( 3 + 4 )") == 10



"""
WatchDogLog - file ``/var/log/watchdog/*.std*``
===============================================
"""

from insights import LogFileOutput, parser
from insights.specs import Specs
from insights.util import deprecated


@parser(Specs.watchdog_logs)
class WatchDogLog(LogFileOutput):
    """
    .. warning::
        This parser is deprecated, please use
        :py:class:`insights.parsers.watchdog.WatchDogLog` instead.

    Class for parsing ``/var/log/watchdog/*.std*`` files.

    Sample Input::

        DEBUG:root:0

        INFO:root:Executing: /usr/bin/sg_persist -n -i -k -d /dev/mapper/mpathb

        DEBUG:root:0   PR generation=0xc2, 4 registered reservation keys follow:
            0xfdfe0001
            0xfdfe0001
            0xfdfe0000
            0xfdfe0000


        DEBUG:root:key fdfe0001 registered with device /dev/mapper/mpathb
        INFO:root:Executing: /usr/bin/sg_turs /dev/mapper/mpathb

        DEBUG:root:0
    """

    def __init__(self, *args, **kwargs):
        deprecated(WatchDogLog, "Please use the :class:`insights.parsers.watchdog.WatchDogLog` instead.", "3.8.0")
        super(WatchDogLog, self).__init__(*args, **kwargs)

    time_format = None



"""
GCPLicenseCodes
===============

This parser reads the output of a command
``curl -H "Metadata-Flavor: Google" "http://metadata.google.internal/computeMetadata/v1/instance/licenses/?recursive=True"``,
which is used to check whether the google cloud instance is a licensed marketplace instance.

For more details, See: https://cloud.google.com/compute/docs/reference/rest/v1/images/get#body.Image.FIELDS.license_code

"""
import json

from insights.core import CommandParser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.gcp_license_codes)
class GCPLicenseCodes(CommandParser):
    """
    Class for parsing the GCP License Codes returned by command
    ``curl -H "Metadata-Flavor: Google" "http://metadata.google.internal/computeMetadata/v1/instance/licenses/?recursive=True"``,


    Typical Output of this command is::

        [{"id": "601259152637613565"}]

    Raises:
        SkipComponent: When content is empty or no parse-able content.
        ParseException: When the json is unable to be parsed

    Attributes:
        ids (list): A list containing the IDs found on the instance
        raw (str): The full JSON of the plan returned by the ``curl`` command

    Examples:
        >>> gcp_licenses.ids == ["601259152637613565"]
        True
        >>> gcp_licenses.raw == [{"id": "601259152637613565"}]
        True
    """

    def parse_content(self, content):
        if not content or 'curl: ' in content[0]:
            raise SkipComponent()
        try:
            license_list = json.loads(content[0])
        except:
            raise ParseException("Unable to parse JSON")

        self.raw = license_list
        self.ids = None
        if len(license_list) >= 1:
            self.ids = [l["id"] for l in license_list]

    def __repr__(self):
        return "ids: {i}, raw: {r}".format(
            i=self.ids, r=self.raw
        )



import os
import pytest
import shlex
import sys
import stat

from insights.core.exceptions import CalledProcessError
from insights.util import subproc


def test_call():
    result = subproc.call('echo -n hello')
    assert result == 'hello'


def test_call_list_of_lists():
    cmd = "echo -n ' hello '"
    cmd = shlex.split(cmd)
    result = subproc.call([cmd, ["grep", "-F", "hello"]])
    assert "hello" in result


def test_call_timeout():
    # Timeouts don't work on OS X
    if sys.platform != "darwin":
        with pytest.raises(CalledProcessError):
            subproc.call('sleep 3', timeout=1)


SCRIPT_CONTENT = """
#!/bin/bash
echo '0123456789'
exit 1
""".strip()


@pytest.fixture(scope="module")
def tmp_script(tmpdir_factory):
    root = str(tmpdir_factory.mktemp("test_subproc"))
    file_path = os.path.join(root, 'tmp.sh')
    with open(file_path, 'w') as fd:
        for line in SCRIPT_CONTENT.splitlines():
            fd.write(line + '\n')
    st = os.stat(file_path)
    os.chmod(file_path, st.st_mode | stat.S_IEXEC)
    return file_path


def test_call_error_output(tmp_script):
    env = os.environ
    env.update(MAX_FAILURE_OUTPUT="6")
    with pytest.raises(CalledProcessError) as cpe:
        subproc.call(tmp_script, env=env)
    assert "012345" in str(cpe)
    assert "0123456" not in str(cpe)



# -*- coding: UTF-8 -*-
from insights.client.phase.v1 import post_update
from mock.mock import patch, MagicMock
from pytest import raises


def patch_insights_config(old_function):
    patcher = patch("insights.client.phase.v1.InsightsConfig",
                    **{"return_value.load_all.return_value.status": False,
                       "return_value.load_all.return_value.unregister": False,
                       "return_value.load_all.return_value.offline": False,
                       "return_value.load_all.return_value.enable_schedule": False,
                       "return_value.load_all.return_value.disable_schedule": False,
                       "return_value.load_all.return_value.analyze_container": False,
                       "return_value.load_all.return_value.display_name": False,
                       "return_value.load_all.return_value.register": False,
                       "return_value.load_all.return_value.diagnosis": None,
                       "return_value.load_all.return_value.list_specs": False,
                       "return_value.load_all.return_value.show_results": False,
                       "return_value.load_all.return_value.check_results": False,
                       "return_value.load_all.return_value.no_upload": False,
                       "return_value.load_all.return_value.module": False})
    return patcher(old_function)


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_legacy_upload_off(insights_config, insights_client):
    """
    Registration is not called when platform upload
    """
    insights_config.return_value.load_all.return_value.legacy_upload = False
    try:
        post_update()
    except SystemExit:
        pass
    insights_client.return_value.register.assert_not_called()
    insights_client.return_value.get_machine_id.assert_called_once()


@patch("insights.client.phase.v1._get_rhsm_identity", return_value=True)
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_legacy_upload_on(insights_config, insights_client, _get_rhsm_identity):
    """
    Registration is processed in legacy_upload=True
    """
    insights_config.return_value.load_all.return_value.legacy_upload = True
    insights_client.return_value.register.return_value = False
    try:
        post_update()
    except SystemExit:
        pass
    insights_client.return_value.register.assert_called_once()


@patch("insights.client.phase.v1._get_rhsm_identity", return_value=True)
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_exit_ok(insights_config, insights_client, _get_rhsm_identity):
    """
    Support collection replaces the normal client run.
    """
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 0


@patch("insights.client.phase.v1._get_rhsm_identity", return_value=True)
@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_no_upload(insights_config, insights_client, _get_rhsm_identity):
    """
    No-upload short circuits this phase
    """
    insights_config.return_value.load_all.return_value.no_upload = True
    try:
        post_update()
    except SystemExit:
        pass
    insights_client.return_value.register.assert_not_called()
    insights_client.return_value.get_machine_id.assert_called_once()


@patch("insights.client.phase.v1.InsightsClient")
@patch_insights_config
def test_post_update_register_machineid(insights_config, insights_client):
    """
    Client run with --register.
        If machine-id found, exit with 0 exit code (don't kill parent)
        Also enable scheduling.
    """
    insights_config.return_value.load_all.return_value.register = True
    insights_client.return_value.get_registration_status = MagicMock(return_value=False)
    insights_client.return_value.register = MagicMock(return_value=False)
    with raises(SystemExit) as exc_info:
        post_update()
    assert exc_info.value.code == 101



import doctest
import pytest

from insights.core.exceptions import SkipComponent
from insights.parsers import tuned
from insights.parsers.tuned import Tuned
from insights.tests import context_wrap

TUNED_OUTPUT = '''
Available profiles:
- balanced
- desktop
- latency-performance
- network-latency
- network-throughput
- powersave
- throughput-performance
- virtual-guest
- virtual-host
Current active profile: virtual-guest
'''.strip()

TUNED_OUTPUT2 = '''
Available profiles:
- balanced
- desktop
- latency-performance
- network-latency
- network-throughput
- powersave
- throughput-performance
- virtual-guest
- virtual-host
It seems that tuned daemon is not running, preset profile is not activated.
Preset profile: virtual-guest
'''.strip()

TUNED_OUTPUT3 = '''
Available profiles:
- balanced                    - General non-specialized tuned profile
- desktop                     - Optimize for the desktop use-case
- hpc-compute                 - Optimize for HPC compute workloads
- latency-performance         - Optimize for deterministic performance at the cost of increased power consumption
- network-latency             - Optimize for deterministic performance at the cost of increased power consumption, focused on low latency network performance
- network-throughput          - Optimize for streaming network throughput, generally only necessary on older CPUs or 40G+ networks
- powersave                   - Optimize for low power consumption
- sap-netweaver               - Optimize for SAP NetWeaver
- throughput-performance      - Broadly applicable tuning that provides excellent performance across a variety of common server workloads
- virtual-guest               - Optimize for running inside a virtual guest
- virtual-guest-vmware
- virtual-host                - Optimize for running KVM guests
Current active profile: virtual-guest-vmware
'''.strip()

TUNED_OUTPUT4 = '''
'''.strip()


def test_active_profile():
    tuned_output = Tuned(context_wrap(TUNED_OUTPUT))
    assert len(tuned_output.get('available')) == 9
    assert tuned_output.get('active') == 'virtual-guest'
    assert tuned_output.get('available') == ['balanced',
                                             'desktop',
                                             'latency-performance',
                                             'network-latency',
                                             'network-throughput',
                                             'powersave',
                                             'throughput-performance',
                                             'virtual-guest',
                                             'virtual-host']


def test_preset_profile():
    tuned_output = Tuned(context_wrap(TUNED_OUTPUT2))
    assert len(tuned_output.get('available')) == 9
    assert tuned_output.get('preset') == 'virtual-guest'
    assert tuned_output.get('available') == ['balanced',
                                             'desktop',
                                             'latency-performance',
                                             'network-latency',
                                             'network-throughput',
                                             'powersave',
                                             'throughput-performance',
                                             'virtual-guest',
                                             'virtual-host']


def test_tuned_profile():
    tuned_output = Tuned(context_wrap(TUNED_OUTPUT3))
    assert len(tuned_output.get('available')) == 12
    assert tuned_output.get('preset') is None
    assert tuned_output.get('active') == 'virtual-guest-vmware'
    assert 'sap-netweaver' in tuned_output.get('available')
    assert 'virtual-guest-vmware' in tuned_output.get('available')

    with pytest.raises(SkipComponent):
        Tuned(context_wrap(''))


def test_doc_example():
    env = {'tuned': Tuned(context_wrap(TUNED_OUTPUT))}
    failed, total = doctest.testmod(tuned, globs=env)
    assert failed == 0



"""
NeutronOVSAgentLog - file ``/var/log/neutron/openvswitch-agent.log``
====================================================================

Parser plugin for parsing the log file for Neutron OVS Agent

Typical content of ``openvswitch-agent.log`` file is::

    2016-11-09 14:39:25.343 3153 INFO neutron.common.config [-] Logging enabled!
    2016-11-09 14:39:25.343 3153 INFO neutron.common.config [-] /usr/bin/neutron-openvswitch-agent version 9.1.0
    2016-11-09 14:39:25.347 3153 WARNING oslo_config.cfg [-] Option "rabbit_hosts" from group "oslo_messaging_rabbit" is deprecated for removal.  Its value may be silently ignored in the future.
    2016-11-09 14:39:25.348 3153 WARNING oslo_config.cfg [-] Option "rabbit_password" from group "oslo_messaging_rabbit" is deprecated for removal.  Its value may be silently ignored in the future.
    2016-11-09 14:39:25.348 3153 WARNING oslo_config.cfg [-] Option "rabbit_userid" from group "oslo_messaging_rabbit" is deprecated for removal.  Its value may be silently ignored in the future.
    2016-11-09 14:39:25.352 3153 INFO ryu.base.app_manager [-] loading app neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native.ovs_ryuapp
    2016-11-09 14:39:27.171 3153 INFO ryu.base.app_manager [-] loading app ryu.app.ofctl.service
    2016-11-09 14:39:27.190 3153 INFO ryu.base.app_manager [-] loading app ryu.controller.ofp_handler
    2016-11-09 14:39:27.209 3153 INFO ryu.base.app_manager [-] instantiating app neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native.ovs_ryuapp of OVSNeutronAgentRyuApp
    2016-11-09 14:39:27.210 3153 INFO ryu.base.app_manager [-] instantiating app ryu.controller.ofp_handler of OFPHandler
    2016-11-09 14:39:27.210 3153 INFO ryu.base.app_manager [-] instantiating app ryu.app.ofctl.service of OfctlService
    2016-11-09 14:39:28.255 3153 INFO oslo_rootwrap.client [-] Spawned new rootwrap daemon process with pid=3925
    2016-11-09 14:39:29.376 3153 INFO neutron.plugins.ml2.drivers.openvswitch.agent.openflow.native.ovs_bridge [-] Bridge br-int has datapath-ID 0000c2670b638c45
"""

from .. import LogFileOutput, parser
from insights.specs import Specs


@parser(Specs.neutron_ovs_agent_log)
class NeutronOVSAgentLog(LogFileOutput):
    """Class for parsing ``/var/log/neutron/openvswitch-agent.log`` file.

    .. note::
        Please refer to its super-class :class:`insights.core.LogFileOutput`
    """
    pass



from pytest import mark

from insights.client.config import InsightsConfig
from insights.cleaner import Cleaner


MAC_cases = """
0c:1a:2b:3c:4d:5e           # GOOD
0d-1A-2B-3C-4D-5E           # GOOD
0e:1A:2B:3C:4D:5E           # GOOD
3f:1a-2b:3c:4d:5e           # NG
001a2b3c4d5e                # NG
00:1a:2b:3c:4d              # NG
00:1a:2b:3c:4d:5e:6f        # NG
00:1G:2B:3C:4D:5E           # NG
abc00:1a:2b:3c:4d:5e        # NG
00:1a:2b:3c:4d:5e-xyz"      # NG

Valid MAC 00:1a:2B:3c:4D:5E in lowercase/uppercase  # GOOD
Valid hyphenated 0d-1A-2B-3C-4D-5E                  # GOOD == line-2
Broadcast address FF:FF:FF:FF:FF:FF                 # NG
00:00:00:00:00:00 null address                      # NG
Multicast example 01:00:5E:1a:34:56                 # GOOD
f1:08:5E:12:34:a6 multipule 01:08:5E:12:34:56       # GOOD
Invalid mixed separators 00:1A-2B:3C:4D:5E          # NG
Invalid character 00:1A:2B:3C:4D:5G,                # NG
No separators 001A2B3C4D5E                          # NG
Edge case:MAC=00:11:22:33:44:55=end"                # GOOD
""".strip().splitlines()

MAC_cases_keep_same = """
fe:ab:04:ff:76:4b dev enp3s0 lladdr router STALE - line 1
FE:ab:04:FF:76:4B dev enp3s0 lladdr router STALE - line 2
dev enp3s0 lladdr router fe:ab:04:FF:76:4b
""".strip().splitlines()


@mark.parametrize("obfuscation_list", [[], ['mac']])
def test_obfuscate_mac(obfuscation_list):
    c = InsightsConfig(obfuscation_list=obfuscation_list)
    pp = Cleaner(c, {})
    mac_changed = 0
    ret0 = pp.clean_content(MAC_cases)
    for idx, line in enumerate(ret0):
        org_line = MAC_cases[idx]
        if line != org_line:
            mac_changed += 1
    if obfuscation_list:
        assert mac_changed == len([l for l in MAC_cases if 'GOOD' in l])
    else:
        assert mac_changed == 0
    # No obfuscated address will be obfuscated:
    ret1 = pp.clean_content(ret0)
    assert ret1 == ret0


@mark.parametrize("obfuscation_list", [[], ['mac']])
def test_obfuscate_mac_the_same(obfuscation_list):
    c = InsightsConfig(obfuscation_list=obfuscation_list)
    pp = Cleaner(c, {})
    mac_changed = 0
    ret0 = pp.clean_content(MAC_cases_keep_same)
    for idx, line in enumerate(ret0):
        org_line = MAC_cases_keep_same[idx]
        if line != org_line:
            mac_changed += 1
    if obfuscation_list:
        assert mac_changed == len(MAC_cases_keep_same)
        # Three MACs are obfuscated
        obf_macs = pp.obfuscate.get('mac').mapping()
        assert len(obf_macs) == 3
        # But they are all the same - 1: in result
        ret_mac = obf_macs[0].get('obfuscated').lower()
        # But they are all the same - 2: in mapping
        for mac in obf_macs:
            assert ret_mac == mac['obfuscated'].lower()
    else:
        assert mac_changed == 0
    # No obfuscated address will be obfuscated:
    ret1 = pp.clean_content(ret0)
    assert ret1 == ret0


def test_obfuscate_mac_no_obfuscate():
    c = InsightsConfig(obfuscation_list=['mac'])
    pp = Cleaner(c, {})
    # no_obfuscate=['mac'] will not obfuscate any MAC address
    ret = pp.clean_content(MAC_cases, no_obfuscate=['mac'])
    assert ret == MAC_cases



import doctest

from insights.parsers import mdatp_managed
from insights.parsers.mdatp_managed import MdatpManaged
from insights.tests import context_wrap

MDATP_MANAGED_CONFIG = """
{
   "exclusionSettings":{
     "exclusions":[
        {
           "$type":"excludedPath",
           "isDirectory":true,
           "path":"/home/*/git<EXAMPLE DO NOT USE>",
           "scopes": [
              "epp"
           ]
        },
        {
           "$type":"excludedPath",
           "isDirectory":true,
           "path":"/run<EXAMPLE DO NOT USE>",
           "scopes": [
              "global"
           ]
        },
        {
           "$type":"excludedPath",
           "isDirectory":false,
           "path":"/var/log/system.log<EXAMPLE DO NOT USE><EXCLUDED IN ALL SCENARIOS>",
           "scopes": [
              "epp", "global"
           ]
        },
        {
           "$type":"excludedFileExtension",
           "extension":".pdf<EXAMPLE DO NOT USE>",
           "scopes": [
              "epp"
           ]
        },
        {
           "$type":"excludedFileName",
           "name":"/bin/cat<EXAMPLE DO NOT USE><NO SCOPE PROVIDED - GLOBAL CONSIDERED>"
        }
     ],
     "mergePolicy":"admin_only"
   }
}
""".strip()


def test_mdatp():
    mdatp = MdatpManaged(context_wrap(MDATP_MANAGED_CONFIG))
    assert "exclusionSettings" in mdatp
    assert "exclusions" in mdatp["exclusionSettings"]
    assert mdatp["exclusionSettings"]["exclusions"][0]["isDirectory"] is True


def test_mdatp_doc():
    env = {
        "mdatp": MdatpManaged(context_wrap(MDATP_MANAGED_CONFIG))
    }
    failed_count, _ = doctest.testmod(mdatp_managed, globs=env)
    assert failed_count == 0



"""
transparent_hugepage sysfs settings
===================================

Module for parsing the sysfs settings for transparent_hugepage:

ThpUseZeroPage - file ``/sys/kernel/mm/transparent_hugepage/use_zero_page``
---------------------------------------------------------------------------

Gets the contents of /sys/kernel/mm/transparent_hugepage/use_zero_page, which is either 0 or 1.

Sample input::

    0

Examples:

    >>> shared[ThpUseZeroPage].use_zero_page
    0


ThpEnabled - file ``/sys/kernel/mm/transparent_hugepage/enabled``
-----------------------------------------------------------------

Gets the contents of  /sys/kernel/mm/transparent_hugepage/enabled, which is something like
`always [madvise] never` where the active value is in brackets.

If no option is active (that should never happen), `active_option` will contain None.

Sample input::

    always [madvise] never

Examples:

    >>> shared[ThpEnabled].line
    always [madvise] never
    >>> shared[ThpEnabled].active_option
    madvise

"""

from .. import Parser, parser
from insights.specs import Specs


@parser(Specs.thp_use_zero_page)
class ThpUseZeroPage(Parser):
    """
    Gets the contents of /sys/kernel/mm/transparent_hugepage/use_zero_page, which is either 0 or 1.

    Attributes:
        use_zero_page (str): The setting, should be 0 or 1.
    """
    def parse_content(self, content):
        self.use_zero_page = " ".join(content).strip()


@parser(Specs.thp_enabled)
class ThpEnabled(Parser):
    """
    Gets the contents of  /sys/kernel/mm/transparent_hugepage/enabled, which is something like
    `always [madvise] never` where the active value is in brackets.
    If no option is active (that should never happen), `active_option` will contain None.

    Attributes:
        line (str): Contents of the input file.
        active_option (str): The active option for transparent huge pages, or `None` if not present.
    """
    def parse_content(self, content):
        self.line = " ".join(content).strip()
        self.active_option = None
        for w in self.line.split():
            if w.startswith("[") and w.endswith("]"):
                self.active_option = w[1:-1]



"""
HttpdConf - files ``/etc/httpd/conf/httpd.conf`` and ``/etc/httpd/conf.d/*``
============================================================================
"""
import string

from insights.core import ConfigParser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsr import (Char, EOF, EOL, EndTagName, Forward, FS, GT, InSet, Literal,
                            LT, Letters, Lift, LineEnd, Many, Number, OneLineComment,
                            PosMarker, QuotedString, StartTagName, String, WS, WSChar,
                            skip_none)
from insights.parsr.query import Directive, Entry, Section
from insights.specs import Specs


class DocParser(object):
    def __init__(self, ctx):
        self.ctx = ctx

        Complex = Forward()
        Comment = (WS >> OneLineComment("#")).map(lambda x: None)

        Cont = Char("\\") + EOL
        First = InSet(string.ascii_letters + "_/")
        Rest = String(string.ascii_letters + "_/" + string.digits)
        FirstRest = (First + Rest).map("".join)
        Name = (FirstRest << (Many(WSChar) + Cont)) | FirstRest

        Num = Number & (WSChar | LineEnd)

        StartName = WS >> PosMarker(StartTagName(Letters)) << WS
        EndName = WS >> EndTagName(Letters, ignore_case=True) << WS

        AttrStart = Many(WSChar)
        AttrEnd = (Many(WSChar) + Cont) | Many(WSChar)

        BareAttr = String(set(string.printable) - (set(string.whitespace) | set("<>'\"")))
        OpAttr = (Literal("!=") | Literal("<=") | Literal(">=") | InSet("<>")) & WSChar + BareAttr
        EmptyAttr = String('"\'', min_length=2)
        Attr = AttrStart >> (Num | QuotedString.map(self.remove_cont) | OpAttr | BareAttr | EmptyAttr) << AttrEnd
        Attrs = Many(Attr)

        StartTag = (WS + LT) >> (StartName + Attrs) << (GT + WS)
        EndTag = (WS + LT + FS) >> EndName << (GT + WS)

        Simple = WS >> (Lift(self.to_directive) * PosMarker(Name) * Attrs) << WS
        Stanza = Simple | Complex | Comment | Many(WSChar | EOL, lower=1).map(lambda x: None)
        Complex <= (Lift(self.to_section) * StartTag * Many(Stanza).map(skip_none)) << EndTag
        Doc = Many(Stanza).map(skip_none)

        self.Top = Doc + EOF

    def remove_cont(self, val):
        return "".join([x.strip().strip("\\") for x in val.split("\n")])

    def typed(self, val):
        try:
            v = val.lower()
            if v in ("on", "yes", "true"):
                return True
            if v in ("off", "no", "false"):
                return False
        except:
            pass
        return val

    def to_directive(self, name, attrs):
        attrs = attrs if len(attrs) > 1 else [self.typed(a) for a in attrs]
        return Directive(name=name.value, attrs=attrs, lineno=name.lineno,
                         src=self.ctx)

    def to_section(self, tag, children):
        name, attrs = tag
        attrs = attrs if len(attrs) > 1 else [self.typed(a) for a in attrs]
        return Section(name=name.value, attrs=attrs, children=children,
                       lineno=name.lineno, src=self.ctx)

    def __call__(self, content):
        try:
            return self.Top(content)
        except Exception:
            raise ParseException("There was an exception when parsing one of the httpd config files.")


class HttpdConfBase(ConfigParser):
    """
    Parse the keyword-and-value-but-also-vaguely-XML of an Apache configuration
    file.

    Generally, each line is split on the first space into key and value, leading
    and trailing space being ignored.

    Sample (edited) httpd.conf file::

        ServerRoot "/etc/httpd"
        LoadModule auth_basic_module modules/mod_auth_basic.so
        LoadModule auth_digest_module modules/mod_auth_digest.so

        <Directory />
            Options FollowSymLinks
            AllowOverride None
        </Directory>

        <IfModule mod_mime_magic.c>
        #   MIMEMagicFile /usr/share/magic.mime
            MIMEMagicFile conf/magic
        </IfModule>

        ErrorLog "|/usr/sbin/httplog -z /var/log/httpd/error_log.%Y-%m-%d"

        SSLProtocol -ALL +SSLv3
        #SSLProtocol all -SSLv2

        NSSProtocol SSLV3 TLSV1.0
        #NSSProtocol ALL

        # prefork MPM
        <IfModule prefork.c>
        StartServers       8
        MinSpareServers    5
        MaxSpareServers   20
        ServerLimit      256
        MaxClients       256
        MaxRequestsPerChild  200
        </IfModule>

        # worker MPM
        <IfModule worker.c>
        StartServers         4
        MaxClients         300
        MinSpareThreads     25
        MaxSpareThreads     75
        ThreadsPerChild     25
        MaxRequestsPerChild  0
        </IfModule>

    Examples:

        >>> httpd_conf['ServerRoot'][-1].value
        '/etc/httpd'
        >>> httpd_conf['LoadModule'][0].value
        'auth_basic_module modules/mod_auth_basic.so'
        >>> httpd_conf['LoadModule'][-1].value
        'auth_digest_module modules/mod_auth_digest.so'
        >>> httpd_conf['Directory', '/']['Options'][-1].value
        'FollowSymLinks'
        >>> type(httpd_conf[('IfModule','prefork.c')]) == type({})
        False
        >>> httpd_conf[('IfModule','prefork.c')]['StartServers'][0].value
        8
        >>> 'ThreadsPerChild' in httpd_conf[('IfModule','prefork.c')]
        False
        >>> httpd_conf[('IfModule','worker.c')]['MaxRequestsPerChild'][-1].value
        0
    """
    def __init__(self, *args, **kwargs):
        self.parse = DocParser(self)
        super(HttpdConfBase, self).__init__(*args, **kwargs)

    def parse_doc(self, content):
        if isinstance(content, list):
            content = "\n".join(content)
        result = self.parse(content)[0]
        return Entry(children=result, src=self)


@parser(Specs.httpd_conf)
class HttpdConf(HttpdConfBase):
    pass


@parser(Specs.httpd_conf_scl_httpd24)
class HttpdConfSclHttpd24(HttpdConfBase):
    pass


@parser(Specs.httpd_conf_scl_jbcs_httpd24)
class HttpdConfSclJbcsHttpd24(HttpdConfBase):
    pass



'''
Module responsible for running the --support option for collecting debug information
'''
from __future__ import absolute_import
import logging
import shlex
import re
import os
import requests
import tempfile
import time
import subprocess
from insights import get_nvr
from subprocess import Popen, PIPE, STDOUT

from .constants import InsightsConstants as constants
from .connection import InsightsConnection
from .utilities import machine_id_exists, write_registered_file, write_unregistered_file, write_to_disk

APP_NAME = constants.app_name
logger = logging.getLogger(__name__)


def _legacy_registration_check(api_reg_status):
    # check local registration record
    unreg_date = None
    unreachable = False
    if os.path.isfile(constants.registered_files[0]):
        local_record = 'System is registered locally via .registered file.'
        with open(constants.registered_files[0]) as reg_file:
            local_record += ' Registered at ' + reg_file.readline()
    else:
        local_record = 'System is NOT registered locally via .registered file.'
    if os.path.isfile(constants.unregistered_files[0]):
        with open(constants.unregistered_files[0]) as reg_file:
            local_record += ' Unregistered at ' + reg_file.readline()

    logger.debug('Registration status: %s', api_reg_status)
    if type(api_reg_status) is bool:
        if api_reg_status:
            api_record = 'Insights API confirms registration.'
        else:
            api_record = 'Insights API could not be reached to confirm registration status.'
            unreachable = True
    elif api_reg_status is None:
        api_record = 'Insights API says this machine is NOT registered.'
        api_reg_status = False
    else:
        api_record = 'Insights API says this machine was unregistered at ' + api_reg_status
        unreg_date = api_reg_status
        api_reg_status = False

    return {'messages': [local_record, api_record],
            'status': api_reg_status,
            'unreg_date': unreg_date,
            'unreachable': unreachable}


def registration_check(pconn):
    status = pconn.api_registration_check()
    # Legacy code
    if pconn.config.legacy_upload:
        status = _legacy_registration_check(status)
    if isinstance(status, dict):
        reg_status = status['status']
        if status['unreachable'] is True:
            reg_status = None
    # --- end legacy ---
    else:
        if not status and machine_id_exists() and os.path.exists(constants.registered_files[0]):
            status = True
        reg_status = status
    if reg_status:
        write_registered_file()
    elif reg_status is False:
        write_unregistered_file()
        write_to_disk(constants.machine_id_file, delete=True)
        if pconn.config.legacy_upload:
            status['messages'].append("System unregistered locally via .unregistered file")

    return status


class InsightsSupport(object):
    '''
    Build the support logfile
    '''
    def __init__(self, config):
        self.config = config

    def collect_support_info(self):
        logger.info('Collecting logs...')
        self._support_diag_dump()
        logger.info('Copying Insights logs to archive...')
        log_archive_dir = tempfile.mkdtemp(prefix='/var/tmp/')
        tar_file = os.path.join(log_archive_dir,
                                'insights-client-logs-' +
                                time.strftime('%Y%m%d%H%M%S') +
                                '.tar.gz')
        tar_cmd = 'tar czfS {0} -C {1} .'.format(
            tar_file,
            constants.log_dir)
        subprocess.call(shlex.split(tar_cmd),
                        stderr=subprocess.PIPE)
        logger.info('Support information collected in %s', tar_file)

    def _support_diag_dump(self):
        '''
        Collect log info for debug
        '''
        # check insights config
        cfg_block = []

        logger.info('Insights version: %s', get_nvr())

        if self.config.offline:
            cfg_block.append('Cannot check registration status while offline.')
        else:
            pconn = InsightsConnection(self.config)
            reg_check = registration_check(pconn)
            cfg_block.append('Registration check:')
            if pconn.config.legacy_upload:
                for key in reg_check:
                    cfg_block.append(key + ': ' + str(reg_check[key]))
            else:
                cfg_block.append("status: " + str(reg_check))

        lastupload = 'never'
        if os.path.isfile(constants.lastupload_file):
            with open(constants.lastupload_file) as upl_file:
                lastupload = upl_file.readline().strip()
        cfg_block.append('\nLast successful upload was ' + lastupload)

        cfg_block.append('auto_config: ' + str(self.config.auto_config))
        if self.config.proxy:
            obfuscated_proxy = re.sub(r'(.*)(:)(.*)(@.*)',
                                      r'\1\2********\4',
                                      self.config.proxy)
        else:
            obfuscated_proxy = 'None'
        cfg_block.append('proxy: ' + obfuscated_proxy)

        logger.info('\n'.join(cfg_block))
        logger.info('python-requests: %s', requests.__version__)

        if not self.config.offline:
            succ = pconn.test_connection()
            if succ == 0:
                logger.info('Connection test: PASS\n')
            else:
                logger.info('Connection test: FAIL\n')

        # run commands
        commands = ['uname -a',
                    'cat /etc/redhat-release',
                    'env',
                    'sestatus',
                    'subscription-manager identity',
                    'systemctl cat insights-client.timer',
                    'systemctl cat insights-client.service',
                    'systemctl status insights-client.timer',
                    'systemctl status insights-client.service']
        for cmd in commands:
            logger.info("Running command: %s", cmd)
            try:
                proc = Popen(
                    shlex.split(cmd), shell=False, stdout=PIPE, stderr=STDOUT,
                    close_fds=True)
                stdout, stderr = proc.communicate()
            except OSError as o:
                if 'systemctl' not in cmd:
                    # suppress output for systemctl cmd failures
                    logger.info('Error running command "%s": %s', cmd, o)
            except Exception as e:
                # unknown error
                logger.info("Process failed: %s", e)
            logger.info("Process output: \n%s", stdout)
        # check available disk space for /var/tmp
        tmp_dir = '/var/tmp'
        dest_dir_stat = os.statvfs(tmp_dir)
        dest_dir_size = (dest_dir_stat.f_bavail * dest_dir_stat.f_frsize)
        logger.info('Available space in %s:\t%s bytes\t%.1f 1K-blocks\t%.1f MB',
                    tmp_dir, dest_dir_size,
                    dest_dir_size / 1024.0,
                    (dest_dir_size / 1024.0) / 1024.0)



"""
CephOsdTree
===========

Combiner provides the information about ceph osd tree. It
uses the results of the ``CephOsdTree``, ``CephInsights`` and ``CephOsdTreeText`` parsers.
The order from most preferred to least preferred is ``CephOsdTree``, ``CephInsights``, ``CephOsdTreeText``.

Examples:
    >>> type(cot)
    <class 'insights.combiners.ceph_osd_tree.CephOsdTree'>
    >>> cot['nodes'][0]['children']
    [-7, -3, -5, -9]
"""

from insights.core.plugins import combiner
from insights import LegacyItemAccess
from insights.parsers.ceph_cmd_json_parsing import CephOsdTree as CephOsdTreeParser
from insights.parsers.ceph_insights import CephInsights
from insights.parsers.ceph_osd_tree_text import CephOsdTreeText


@combiner([CephOsdTreeParser, CephInsights, CephOsdTreeText])
class CephOsdTree(LegacyItemAccess):
    """
    Combiner provides the information about ceph osd tree. It
    uses the results of the ``CephOsdTree``, ``CephInsights`` and ``CephOsdTreeText`` parsers.
    The order from most preferred to least preferred is ``CephOsdTree``, ``CephInsights``, ``CephOsdTreeText``.
    """

    def __init__(self, cot, ci, cott):
        if cot:
            self.data = cot
        elif ci:
            self.data = ci.data['osd_tree']
        else:
            self.data = cott



from insights.core.exceptions import SkipComponent
from insights.parsers import repquota
from insights.parsers.repquota import RepquotaAGNPUV
from insights.tests import context_wrap
import doctest
import pytest

REPQUOTAAGNPUV_ERR = """
""".strip()

REPQUOTAAGNPUV = """
*** Report for user quotas on device /dev/sdb
Block grace time: 7days; Inode grace time: 7days
                        Block limits                File limits
User            used    soft    hard  grace    used  soft  hard  grace
----------------------------------------------------------------------
#0        --    5120       0       0      0       4     0     0      0
#1009     +-   61440   51200  102400 1679983770       1     0     0      0
#1010     --       0   51200  102400      0       0     0     0      0
#1011     +-  107520   51200  307200 1680640819       2     0     0      0
#1012     --   51200       0       0      0       1     0     0      0
#1013     --       0   51200  102400      0       0     0     0      0

*** Status for user quotas on device /dev/sdb
Accounting: ON; Enforcement: ON
Inode: #131 (2 blocks, 2 extents)

*** Report for group quotas on device /dev/sdb
Block grace time: 7days; Inode grace time: 7days
                        Block limits                File limits
Group           used    soft    hard  grace    used  soft  hard  grace
----------------------------------------------------------------------
#0        --    5120       0       0      0       4     0     0      0
#1004     --   61440       0       0      0       1     0     0      0
#1005     --   51200  972800 1048576      0       1     0     0      0
#1011     --  107520       0       0      0       2     0     0      0

*** Status for group quotas on device /dev/sdb
Accounting: ON; Enforcement: ON
Inode: #132 (2 blocks, 2 extents)
""".strip()


def test_repquota():
    results = RepquotaAGNPUV(context_wrap(REPQUOTAAGNPUV))
    assert len(results.group_quota.keys()) == 1
    assert len(results.group_quota['/dev/sdb']['quota_info']) == 4
    assert results.group_quota['/dev/sdb']['quota_info'][1] == {'group': '1004', 'status': '--', 'block_used': '61440', 'block_soft': '0', 'block_hard': '0', 'block_grace': '0', 'file_used': '1', 'file_soft': '0', 'file_hard': '0', 'file_grace': '0'}
    assert len(results.user_quota['/dev/sdb']['quota_info']) == 6
    assert results.user_quota['/dev/sdb']['enforcement'] is True


def test_repquota_err():
    with pytest.raises(SkipComponent):
        repquota.RepquotaAGNPUV(context_wrap(REPQUOTAAGNPUV_ERR))


def test_repquota_doc_examples():
    env = {
        'repquota': RepquotaAGNPUV(context_wrap(REPQUOTAAGNPUV))
    }
    failed, total = doctest.testmod(repquota, globs=env)
    assert failed == 0



"""
LruGenEnabled - file ``/sys/kernel/mm/lru_gen/enabled``
=======================================================

Parser to parse the output of file ``/sys/kernel/mm/lru_gen/enabled``
"""

from insights.core import Parser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.lru_gen_enabled)
class LruGenEnabled(Parser):
    """
    The parser for  ``/sys/kernel/mm/lru_gen/enabled`` file.

    The content of ``/sys/kernel/mm/lru_gen/enabled`` file is a hexadecimal number.
    Values , Feature
    ----------------
    0x0000, the multi-gen LRU feature is disabled
    0x0001, the multi-gen LRU feture is enabled
    0x0002, the feature of clearing the accessed bit in leaf page table entries in large batches is enabled
    0x0004, the feature of clearing the accessed bit in non-leaf page table entries as well is enabled
    0x0007, all the features are enabled

    Sample Content::
        0x0004

    Examples:
        >>> type(lru_gen_enabled)
        <class 'insights.parsers.lru_gen_enabled.LruGenEnabled'>
        >>> lru_gen_enabled.enabled
        True
        >>> lru_gen_enabled.features
        4

    Attributes:
        enabled (bool): False means multi-gen LRU feature is disabled,
            otherwise, any of feature is enabled.
        features (int): when enabled is True, checking the features to see
            which features are enabled.

    Raises:
        ParseException: When running into an unparsable line.
        SkipComponent: When file content is empty or has multiple lines.

    """

    def parse_content(self, content):
        if len(content) != 1:
            raise SkipComponent("Input content should only contain one line")

        if not content[0].startswith("0x"):
            raise ParseException("Input content is not a hex number")

        try:
            features = int(content[0], 16)
        except:
            raise ParseException("Input content is not a hex number")

        self.features = features
        self.enabled = features > 0



import doctest
import pytest

from insights.core.exceptions import ParseException, SkipComponent
from insights.parsers import lru_gen_enabled
from insights.parsers.lru_gen_enabled import LruGenEnabled
from insights.tests import context_wrap

LRU_GEN_DISABLED = "0x0000"
LRU_GEN_ENABLED = "0x0004"


def test_lru_gen_enabled():
    lru_gen_enabled = LruGenEnabled(context_wrap(LRU_GEN_DISABLED))
    assert lru_gen_enabled.enabled is False
    assert lru_gen_enabled.features == 0

    lru_gen_enabled = LruGenEnabled(context_wrap(LRU_GEN_ENABLED))
    assert lru_gen_enabled.enabled is True
    assert lru_gen_enabled.features == 4


LRU_GEN_MULTIPLE_LINES = """
line 1
line 2
""".strip()


def test_lru_gen_exceptions():
    with pytest.raises(SkipComponent) as err:
        LruGenEnabled(context_wrap(LRU_GEN_MULTIPLE_LINES))
    assert "Input content should only contain one line" in str(err)

    with pytest.raises(SkipComponent) as err:
        LruGenEnabled(context_wrap(""))
    assert "Input content should only contain one line" in str(err)

    with pytest.raises(ParseException) as err:
        LruGenEnabled(context_wrap("1"))
    assert "Input content is not a hex number" in str(err)

    with pytest.raises(ParseException) as err:
        LruGenEnabled(context_wrap("0xtest"))
    assert "Input content is not a hex number" in str(err)


def test_lru_gen_enabled_doc_examples():
    env = {
        'lru_gen_enabled': LruGenEnabled(context_wrap(LRU_GEN_ENABLED)),
    }
    failed, total = doctest.testmod(lru_gen_enabled, globs=env)
    assert failed == 0



"""
BondDynamicLB - file ``/sys/class/net/bond[0-9]*/bonding/tlb_dynamic_lb``
=========================================================================

This file represent weather the transmit load balancing is enabled or not

tlb_dynamic_lb=1 mode:
    The outgoing traffic is distributed according to the current load.

tlb_dynamic_lb=0 mode:
    The load balancing based on current load is disabled and the load
    is distributed only using the hash distribution.

Typical content of the file is::

        1

Data is modeled as an array of ``BondDynamicLB`` objects

Examples:
    >>> type(tlb_bond)
    <class 'insights.parsers.bond_dynamic_lb.BondDynamicLB'>
    >>> tlb_bond.dynamic_lb_status
    1
    >>> tlb_bond.bond_name
    'bond0'

"""
from insights.core import Parser
from insights.core.exceptions import ParseException, SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.bond_dynamic_lb)
class BondDynamicLB(Parser):
    """
    Models the ``/sys/class/net/bond[0-9]*/bonding/tlb_dynamic_lb`` file.

    0 - Hash based load balancing.

    1 - Load based load balancing.

    Raises:
        SkipComponent: When contents are empty
        ParseException: When contents are invalid
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("No Contents")
        line = content[0].strip()
        self._dynamic_lb_status = None
        self._bond_name = self.file_path.rsplit("/")[-3]

        if line in ['0', '1']:
            self._dynamic_lb_status = int(line)
        else:
            raise ParseException("Unrecognised Values '{b}'".format(b=line))

    @property
    def dynamic_lb_status(self):
        """
        Returns (int): Load balancer type
        """
        return self._dynamic_lb_status

    @property
    def bond_name(self):
        """
        Returns (str): Name of bonding interface
        """
        return self._bond_name



"""
Bond - file ``/proc/net/bonding``
=================================

Provides plugins access to the network bonding information gathered from
all the files starteing with "bond." located in the
``/proc/net/bonding`` directory.

Typical content of ``bond.*`` file is::

    Ethernet Channel Bonding Driver: v3.2.4 (January 28, 2008)

    Bonding Mode: IEEE 802.3ad Dynamic link aggregation
    Transmit Hash Policy: layer2 (0)
    MII Status: up
    MII Polling Interval (ms): 500
    Up Delay (ms): 0
    Down Delay (ms): 0

    802.3ad info
    LACP rate: slow
    Active Aggregator Info:
            Aggregator ID: 3
            Number of ports: 1
            Actor Key: 17
            Partner Key: 1
            Partner Mac Address: 00:00:00:00:00:00

    Slave Interface: eth1
    MII Status: up
    Link Failure Count: 0
    Permanent HW addr: 00:16:35:5e:42:fc
    Aggregator ID: 3

    Slave Interface: eth2
    MII Status: up
    Link Failure Count: 0
    Permanent HW addr: 00:16:35:5e:02:7e
    Aggregator ID: 2

Data is modeled as an array of ``Bond`` objects (``bond`` being a
pattern file specification gathering data from files located in
``/proc/net/bonding``.

Examples:
    >>> type(bond_info)
    <class 'insights.parsers.bond.Bond'>
    >>> bond_info.bond_mode
    '4'
    >>> bond_info.partner_mac_address
    '00:00:00:00:00:00'
    >>> bond_info.slave_interface
    ['eth1', 'eth2']
    >>> bond_info.aggregator_id
    ['3', '3', '2']
    >>> bond_info.xmit_hash_policy
    'layer2'
    >>> bond_info.active_slave
    >>> bond_info.slave_duplex
    ['full', 'full']
    >>> bond_info.slave_speed
    ['1000 Mbps', '1000 Mbps']
"""
from insights.core import Parser
from insights.core.exceptions import ParseException
from insights.core.plugins import parser
from insights.parsers import get_active_lines
from insights.specs import Specs

"""dict: bonding mode parameter string linked to bond type index."""
BOND_PREFIX_MAP = {
    'load balancing (round-robin)': '0',
    'fault-tolerance (active-backup)': '1',
    'fault-tolerance (active-backup) (fail_over_mac active)': '1',
    'load balancing (xor)': '2',
    'fault-tolerance (broadcast)': '3',
    'IEEE 802.3ad Dynamic link aggregation': '4',
    'transmit load balancing': '5',
    'adaptive load balancing': '6'
}


@parser(Specs.bond)
class Bond(Parser):
    """
    Models the ``/proc/net/bonding`` file.

    Currently used information from ``/proc/net/bonding`` includes
    the "bond mode" and "partner mac address".
    """

    def parse_content(self, content):
        self._bond_mode = None
        self._partner_mac_address = None
        self._active_slave = None
        self.xmit_hash_policy = None
        self._arp_polling_interval = None
        self._arp_ip_target = None
        self._mii_polling_interval = None
        self._slave_interface = []
        self._aggregator_id = []
        self._mii_status = []
        self._slave_link_failure_count = []
        self._slave_speed = []
        self._slave_duplex = []
        self._primary_slave = None
        self._up_delay = None
        self._down_delay = None
        self._data = {}

        name_slave = None
        for line in get_active_lines(content):
            if line.startswith("Bonding Mode: "):
                raw_mode = line.split(":", 1)[1].strip()
                self._bond_mode = raw_mode
                self._data['mode'] = self._bond_mode
                if raw_mode in BOND_PREFIX_MAP:
                    self._bond_mode = BOND_PREFIX_MAP[raw_mode]
                    self._data['mode'] = self._bond_mode
                else:
                    raise ParseException("Unrecognised bonding mode '{b}'".format(b=raw_mode))
            elif line.startswith("Partner Mac Address: "):
                self._partner_mac_address = line.split(":", 1)[1].strip()
                self._data['partner_mac'] = self._partner_mac_address
            elif line.startswith("Slave Interface: "):
                name_slave = line.split(":", 1)[1].strip()
                self._slave_interface.append(name_slave)
                self._data[name_slave] = {}
            elif line.strip().startswith("Aggregator ID: "):
                agg_id = line.strip().split(':', 1)[1].strip()
                self._aggregator_id.append(agg_id)
                if name_slave:
                    self._data[name_slave]['aggregator_id'] = agg_id
                else:
                    self._data['aggregator_id'] = agg_id
            elif line.strip().startswith("Transmit Hash Policy"):
                # No need of values in bracket:
                # Integer notification (0), (1), (2) of layer2, layer3+4, layer2+3 resp
                self.xmit_hash_policy = line.split(":", 1)[1].split()[0]
            elif line.strip().startswith("Currently Active Slave"):
                self._active_slave = line.split(":", 1)[1].split()[0]
            elif line.strip().startswith("MII Status: "):
                mii_status = line.strip().split(':', 1)[1].strip()
                self._mii_status.append(mii_status)
                if name_slave:
                    self._data[name_slave]['mii_status'] = mii_status
                else:
                    self._data['mii_status'] = mii_status
            elif line.strip().startswith("Link Failure Count: "):
                link_fail_cnt = line.strip().split(':', 1)[1].strip()
                self._slave_link_failure_count.append(link_fail_cnt)
                if name_slave:
                    self._data[name_slave]['link_fail_cnt'] = link_fail_cnt
            elif line.strip().startswith("Permanent HW addr: "):
                if name_slave:
                    self._data[name_slave]['permanent_hw_addr'] = line.strip().split(':', 1)[1].strip()
            elif line.strip().startswith("Speed: "):
                speed = line.strip().split(':', 1)[1].strip()
                self._slave_speed.append(speed)
                self._data[name_slave]['speed'] = speed
            elif line.strip().startswith("Duplex: "):
                duplex = line.strip().split(':', 1)[1].strip()
                self._slave_duplex.append(duplex)
                self._data[name_slave]['duplex'] = duplex
            elif line.strip().startswith("ARP Polling Interval (ms):"):
                self._arp_polling_interval = line.strip().split(':', 1)[1].strip()
            elif line.strip().startswith("ARP IP target/s (n.n.n.n form):"):
                self._arp_ip_target = line.strip().split(':', 1)[1].strip()
            elif line.strip().startswith("MII Polling Interval (ms):"):
                self._mii_polling_interval = line.strip().split(':', 1)[1].strip()
            elif line.strip().startswith("Primary Slave"):
                self._primary_slave = line.split(":", 1)[1].strip()
            elif line.strip().startswith("Up Delay (ms):"):
                self._up_delay = line.strip().split(':', 1)[1].strip()
            elif line.strip().startswith("Down Delay (ms):"):
                self._down_delay = line.strip().split(':', 1)[1].strip()

    @property
    def bond_mode(self):
        """Returns the bond mode number as a string, or if there is no
        known mapping to a number, the raw "Bonding Mode" value.
        ``None`` is returned if no "Bonding Mode" key is found.
        """
        return self._bond_mode

    @property
    def partner_mac_address(self):
        """Returns the value of the "Partner Mac Address" in the bond
        file if the key/value exists.  If the key is not in the bond
        file, ``None`` is returned.
        """
        return self._partner_mac_address

    @property
    def slave_interface(self):
        """Returns all the slave interfaces of in the bond file wrapped
        a list if the key/value exists.  If the key is not in the
        bond file, ``[]`` is returned.
        """
        return self._slave_interface

    @property
    def aggregator_id(self):
        """Returns all the aggregator id of in the bond file wrapped
        a list if the key/value exists.  If the key is not in the
        bond file, ``[]`` is returned.
        """
        return self._aggregator_id

    @property
    def active_slave(self):
        """Returns the active slave of the "Currently Active Slave" in the bond
        file if key/value exists. If the key is not in the bond file, ``None``
        is returned.
        """
        return self._active_slave

    @property
    def mii_status(self):
        """Returns the master and all the slaves "MII Status" value in the bond file wrapped
        a list if the key/value exists.  If the key is not in the
        bond file, ``[]`` is returned.
        """
        return self._mii_status

    @property
    def slave_link_failure_count(self):
        """Returns all the slaves "Link Failure Count" value in the bond file wrapped
        a list if the key/value exists.  If the key is not in the
        bond file, ``[]`` is returned.
        """
        return self._slave_link_failure_count

    @property
    def slave_speed(self):
        """Returns all the slaves "Speed" value in the bond file wrapped
        a list if the key/value exists.  If the key is not in the
        bond file, ``[]`` is returned.
        """
        return self._slave_speed

    @property
    def slave_duplex(self):
        """Returns all the slave "Duplex" value in the bond file wrapped
        a list if the key/value exists.  If the key is not in the
        bond file, ``[]`` is returned.
        """
        return self._slave_duplex

    @property
    def arp_polling_interval(self):
        """Returns the arp polling interval as a string. ``None`` is returned
        if no "ARP Polling Interval (ms)" key is found.
        """
        return self._arp_polling_interval

    @property
    def arp_ip_target(self):
        """Returns the arp ip target as a string. ``None`` is returned
        if no "ARP IP target/s (n.n.n.n form)" key is found.
        """
        return self._arp_ip_target

    @property
    def mii_polling_interval(self):
        """Returns the mii polling interval as a string. ``None`` is returned
        if no "MII Polling Interval (ms)" key is found.
        """
        return self._mii_polling_interval

    @property
    def primary_slave(self):
        """Returns the "Primary Slave" in the bond file if key/value exists.
        If the key is not in the bond file, ``None`` is returned.
        """
        return self._primary_slave

    @property
    def up_delay(self):
        """Returns the "Up Delay" in the bond file if key/value exists.
        If the key is not in the bond file, ``None`` is returned.
        """
        return self._up_delay

    @property
    def down_delay(self):
        """Returns the "Down Delay" in the bond file if key/value exists.
        If the key is not in the bond file, ``None`` is returned.
        """
        return self._down_delay

    @property
    def data(self):
        """Returns all the details of bond interface and corresponding slave details
        on sucess else it will return empty ``{}``.
        """
        return self._data



"""
Rpm -V Packages Parsers - command ``/bin/rpm -V <packages>``
============================================================

Parsers provided in this module includes:

RpmVPackage - command ``/bin/rpm -V <package>``
-----------------------------------------------
"""

from insights.core import CommandParser
from insights.core.exceptions import SkipComponent
from insights.core.plugins import parser
from insights.specs import Specs


@parser(Specs.rpm_V_package)
class RpmVPackage(CommandParser):
    """
    Class for parsing ``/bin/rpm -V <package>`` command.

    Attributes:
        package_name (String): The name of package
        discrepancies (list of dictionaries): Every dictionary contains information about one entry
        error_lines (list of strings): The error messages from the command `rpm -V <package>`

    Raises:
        SkipComponents: When a package is not installed and no error is reported.

    Sample output of this command is::

        ..?......  c /etc/sudoers
        ..?......    /usr/bin/sudo
        ..?......    /usr/bin/sudoreplay
        missing     /var/db/sudo/lectured (Permission denied)

    OR::

        error: rpmdb: BDB0113 Thread/process 259/139 failed: BDB1507 Thread died in Berkeley DB library
        error: db5 error(-30973) from dbenv->failchk: BDB0087 DB_RUNRECOVERY: Fatal error, run database recovery
        error: cannot open Packages index using db5 - (-30973)

    Examples:
        >>> type(rpm_v_pkg)
        <class 'insights.parsers.rpm_v_packages.RpmVPackage'>
        >>> rpm_v_pkg.package_name
        'sudo'
        >>> len(rpm_v_pkg.discrepancies)
        4
        >>> sorted(rpm_v_pkg.discrepancies[0].items())
        [('attributes', '..?......'), ('file', '/etc/sudoers'), ('line', '..?......  c /etc/sudoers'), ('mark', 'c')]
        >>> "error: db5 error(-30973) from dbenv->failchk: BDB0087 DB_RUNRECOVERY: Fatal error, run database recovery" in rpm_v_pkg.error_lines
        True
    """

    def parse_content(self, content):
        self.error_lines = []
        self.discrepancies = []
        self.package_name = self.file_name.split('_')[-1] if self.file_name else None

        for line in content:
            if line.startswith("error: "):
                self.error_lines.append(line)
                continue

            line_parts = line.split()
            if not line_parts:
                continue

            entry = {}
            if "package" in line_parts[0]:
                if not self.error_lines:
                    # Skip the parser only when:
                    # - The package is not installed
                    # - No error is reported
                    raise SkipComponent("Package is not installed")

            if "missing" in line_parts[0]:
                entry = {"line": line.strip(), "attributes": None, "mark": None, "file": None}
            elif len(line_parts) == 3:
                entry = {
                    "line": line.strip(),
                    "attributes": line_parts[0],
                    "mark": line_parts[1],
                    "file": line_parts[2],
                }
            elif len(line_parts) == 2:
                entry = {
                    "line": line.strip(),
                    "attributes": line_parts[0],
                    "mark": None,
                    "file": line_parts[1],
                }
            if entry:
                self.discrepancies.append(entry)

        if not self.discrepancies and not self.error_lines:
            raise SkipComponent("Empty result")



import pytest

from insights.parsers.watchdog_logs import WatchDogLog
from insights.tests import context_wrap

WATCHDOG_LOG = """
DEBUG:root:0

INFO:root:Executing: /usr/bin/sg_persist -n -i -k -d /dev/mapper/mpathb

DEBUG:root:0   PR generation=0xc2, 4 registered reservation keys follow:
    0xfdfe0001
    0xfdfe0001
    0xfdfe0000
    0xfdfe0000
""".strip()


def test_watchdog():
    WatchDogLog.last_scan('register_line', 'registered reservation keys')
    watch_log = WatchDogLog(context_wrap(WATCHDOG_LOG))
    assert watch_log.register_line is not None
    assert watch_log.register_line.get('raw_message') == 'DEBUG:root:0   PR generation=0xc2, 4 registered reservation keys follow:'


def test_exception():
    watch_log = WatchDogLog(context_wrap(WATCHDOG_LOG))
    with pytest.raises(RuntimeError):
        list(watch_log.get_after('2013-1-1'))



"""
Ansible - Parsers relates to Ansible
====================================

Below Parser is included in this module

AnsibleTelemetry - command "/usr/share/ansible/telemetry/telemetry.py"
----------------------------------------------------------------------
"""

import json

from insights import CommandParser, parser
from insights.core.exceptions import SkipComponent, ParseException
from insights.specs import Specs


@parser(Specs.ansible_telemetry)
class AnsibleTelemetry(CommandParser, list):
    """
    Parse the output of command "/usr/share/ansible/telemetry/telemetry.py".

    Sample output of the command is in NDJSON format::

        {"collections":{"ansible.builtin":{"resources":{"action":{"ansible.builtin.command":13}},"version":"*"}},"ansible_core":{"version":"2.18.9rc1"}}
        {"collections":{"ansible.builtin":{"resources":{"action":{"ansible.builtin.command":14}},"version":"*"}},"ansible_core":{"version":"2.19.9"}}

    Raises:
        SkipComponent: when nothing is parsed.
        ParseException: when any line is not parsable for JSON.

    Examples:
        >>> type(ansible_telemetry)
        <class 'insights.parsers.ansible.AnsibleTelemetry'>
        >>> ansible_telemetry[0]['collections']['ansible.builtin']['version'] == '*'
        True
        >>> ansible_telemetry[1]['ansible_core']['version'] == '2.19.9'
        True
    """

    def parse_content(self, content):
        if not content:
            raise SkipComponent("Empty output.")

        for line in content:
            line = line.strip()
            if line:
                try:
                    line_json = json.loads(line)
                except Exception:
                    raise ParseException("Invalid line: {0}".format(line))
                self.append(line_json) if line_json else None

        if len(self) == 0:  # pragma: no cover
            raise SkipComponent("Nothing parsable.")
